import {select} from 'd3-selection';

import {drawErrorAlert, drawInfoAlert} from 'd3render/alerts';
import {drawLoadingIndicator} from 'd3render/loading-indicator';

import {getAvailableGWDVTimeSeries, getAvailableGWDVTimeSeriesIdsForParameterCode} from 'ml/selectors/daily-value-time-series-selector';
import {Actions} from 'ml/store/daily-value-time-series';

import {drawGraphBrush} from './graph-brush';
import {drawGraphControls} from './graph-controls';
import {drawTimeSeriesLegend} from './legend';
import {drawTimeSeriesGraph} from './time-series-graph';
import {drawTooltipCursorSlider} from './tooltip';


/*
 * Returns the set of time series for the best parameter code.
 * @return Array - of ts to use initially.
 */
const getInitialParameterCode = function(availableTimeSeries) {
    const bestTimeSeries = availableTimeSeries.filter(ts => ts.best);
    return bestTimeSeries.length ? bestTimeSeries[0].parameterCode : availableTimeSeries[0].parameterCode;
};

/*
 * Creates the daily value hydrograph component on the DOM element node for siteno and state
 * information that is stored in the Redux store.
 * @param {Object} store - Redux store
 * @param {Object} node - DOM element parent for this component
 * @param {String} siteno - the site number of the monitoring location.
 */
export const attachToNode = function(store,
                                      node,
                                      {
                                          siteno,
                                          agencyCd
                                      } = {}) {
    const nodeElem = select(node);
    if (!siteno) {
        nodeElem.call(drawErrorAlert, {
            title: 'Must specify monitoring location ID',
            body: ''
        });
        return;
    }

    const monitoringLocationId = `${agencyCd}-${siteno}`;
    const loadingIndicator = nodeElem.select('.loading-indicator-container')
        .call(drawLoadingIndicator, {
            showLoadingIndicator: true
        });
    const fetchAvailableDVTimeSeries = store.dispatch(Actions.retrieveAvailableDVTimeSeries(monitoringLocationId));
    fetchAvailableDVTimeSeries.then(() => {
        const availableTimeSeries = getAvailableGWDVTimeSeries(store.getState());
        if (availableTimeSeries.length) {
            const initialParameterCode = getInitialParameterCode(availableTimeSeries);
            const initialTimeSeriesIds =
                getAvailableGWDVTimeSeriesIdsForParameterCode(initialParameterCode)(store.getState());
            const fetchDVTimeSeries =
                Object.values(initialTimeSeriesIds)
                    .filter(tsId => tsId)
                    .map(tsId => store.dispatch(Actions.retrieveDVTimeSeries(monitoringLocationId, tsId)));
            Promise.allSettled(fetchDVTimeSeries).then(() => {
                store.dispatch(Actions.setCurrentDVTimeSeriesIds(
                    initialTimeSeriesIds.min, initialTimeSeriesIds.mean, initialTimeSeriesIds.max));
                loadingIndicator.call(drawLoadingIndicator, {showLoadingIndicator: false});
                let graphContainer = nodeElem.select('.graph-container');
                graphContainer
                    .call(drawGraphControls, store, monitoringLocationId, initialParameterCode)
                    .call(drawTimeSeriesGraph, store)
                    .call(drawTooltipCursorSlider, store)
                    .call(drawGraphBrush, store)
                    .append('div')
                        .classed('dv-legend-container', true)
                        .call(drawTimeSeriesLegend, store);
            });

        } else {
            loadingIndicator.call(drawLoadingIndicator, {showLoadingIndicator: false});
            drawInfoAlert(nodeElem, {
                title: 'Why don\'t I see a groundwater graph?',
                body: 'No groundwater level statistical daily data has been reported for this location.'
            });
        }
    });
};
