import {axisBottom, axisLeft, axisRight} from 'd3-axis';
import memoize from 'fast-memoize';
import {createSelector} from 'reselect';

import config from 'ui/config';

import {generateTimeTicks} from 'd3render/tick-marks';
import {getIVParameter} from 'ml/selectors/hydrograph-data-selector';

import {getSecondaryYAxisTickDetails, getPrimaryYAxisTickDetails} from './domain';
import {getLayout} from './layout';
import {getXScale, getBrushXScale, getYScale} from './scales';
import {getPrimaryParameter} from './time-series-data';


const createXAxis = function(xScale) {
    const [startMillis, endMillis] = xScale.domain();
    const ticks = generateTimeTicks(startMillis, endMillis, config.locationTimeZone);

    return axisBottom()
        .scale(xScale)
        .tickValues(ticks.dates)
        .tickSizeOuter(0)
        .tickFormat(ticks.format);
};

/**
 * Create an x and y axis for hydrograph
 * @param {Object} xScale      D3 Scale object for the x-axis
 * @param {Object} primaryYScale      D3 Scale object for the y-axis
 * @param {Object} primaryTickDetails - Object which has information about tick values and format.
 * @param {Number} yTickSize   Size of inner ticks for the y-axis
 * @return {Object} {xAxis, yAxis} - D3 Axis
 */
const createAxes = function(xScale, primaryYScale, secondaryYScale, primaryTickDetails, secondaryYTickDetails, yTickSize) {
    // Create x-axis
    const xAxis = createXAxis(xScale);

    // Create y-axis
    const primaryYAxis = axisLeft()
        .scale(primaryYScale)
        .tickValues(primaryTickDetails.tickValues)
        .tickFormat(primaryTickDetails.tickFormat)
        .tickSizeInner(yTickSize)
        .tickPadding(3)
        .tickSizeOuter(0);

    const secondaryYAxis = axisRight()
        .scale(secondaryYScale)
        .tickValues(secondaryYTickDetails.tickValues)
        .tickFormat(secondaryYTickDetails.tickFormat)
        .tickSizeInner(yTickSize)
        .tickPadding(3)
        .tickSizeOuter(0);

    return {xAxis, primaryYAxis, secondaryYAxis};
};

/**
 * Returns a Redux selector function that returns the brush x axis
 */
export const getBrushXAxis = createSelector(
    getBrushXScale,
    xScale => createXAxis(xScale)
);

/**
 * Returns a Redux Selection that returns an object with xAxis, yAxis, and secondaryYAxis properties
 */
export const getAxes = memoize(graphKind => createSelector(
    getXScale(graphKind, 'current'),
    getYScale(graphKind, 'primary'),
    getYScale(graphKind, 'secondary'),
    getPrimaryYAxisTickDetails,
    getSecondaryYAxisTickDetails,
    getLayout(graphKind),
    getPrimaryParameter,
    getIVParameter('secondary'),
    (xScale, primaryYScale, secondaryYScale, primaryYTickDetails, secondaryYTickDetails, layout, primaryParameter, secondaryParameter) => {
        return {
            ...createAxes(
                xScale,
                primaryYScale,
                secondaryYScale,
                primaryYTickDetails,
                secondaryYTickDetails,
                -layout.width + layout.margin.right
            ),
            layout: layout,
            primaryYTitle: primaryParameter ? primaryParameter.unit : '',
            secondaryParameter: secondaryParameter ? secondaryParameter.parameterCode : '',
            secondaryYTitle: secondaryParameter ? secondaryParameter.unit : ''
        };
    }
));
