import isEqual from 'lodash/isEqual';
import uniqWith from 'lodash/uniqWith';
import {DateTime} from 'luxon';
import {createSelector} from 'reselect';

import config from 'ui/config';

import {getSelectedGroundwaterLevels} from 'ml/selectors/groundwater-level-field-visits-selector';

const GW_LEVEL_RADIUS = 7;
const GW_LEVEL_CLASS = 'gw-level-point';

const TIME_FORMAT = {
    'minute': 'y-MM-dd HH:mmZZ',
    'hour': 'y-MM-dd HHZZ',
    'day': 'y-MM-dd',
    'month': 'y-MM',
    'year': 'y'
};

const APPROVAL_DATA = {
    'Approved': {label: 'Approved', class: 'approved'},
    'Provisional': {label: 'Provisional', class: 'provisional'},
    'Revised': {label: 'Revised', class: 'revised'}
};

const getDetailsForApprovalCode = function(approvals) {
    const approval = approvals.find(approval => approval in APPROVAL_DATA);
    if (approval) {
        return APPROVAL_DATA[approval];
    } else {
        return APPROVAL_DATA['Provisional'];
    }
};

/*
 * Returns a selector function that returns the groundwater levels that will be visible
 * on the hydrograph
 * @return {Function} which returns an {Array} of groundwater level object with properties:
 *      @prop {Float} value
 *      @prop {String} displayValue - the string to use for display with precision retained.
 *      @prop {Number} dateTime
 *      @prop {Array of String} classes - a class that can be used to style this point
 *      @prop {String} label - a human readable label for this kind of point
 *      @prop {Number} radius - used to draw the circle marker
 */
export const getGroundwaterLevelPoints = createSelector(
    getSelectedGroundwaterLevels,
    gwLevels => {
        if (!gwLevels) {
            return [];
        }
        return gwLevels.values
            .filter(value => value.result)
            .map(value => {
                const approvalDetails = getDetailsForApprovalCode(value.approvals);
                return {
                    value: parseFloat(value.result),
                    displayValue: value.result,
                    dateTime: value.epochTime,
                    classes: [GW_LEVEL_CLASS, approvalDetails.class],
                    label: approvalDetails.label,
                    radius: GW_LEVEL_RADIUS
                };
            });
    }
);

/*
 * Returns a selector function which returns a two element Array for the
 * value range for the groundwater levels.
 * @return {Function} return an {Array of Number} - [min, max]
 */
export const getGroundwaterLevelDataRange = createSelector(
    getGroundwaterLevelPoints,
    points => {
        if (!points || !points.length) {
            return null;
        }

        const values = points.map(data => data.value);
        return [Math.min(...values), Math.max(...values)];
    }
);

/*
 * Returns a selector function which returns the unique classes/label/radius for the gw level points
 * @return {Function} which returns an {Array of Object} with the following properties:
 *      @prop {Array of String} classes
 *      @prop {String} label
 *      @prop {Number} radius
 */
export const getUniqueGWKinds = createSelector(
    getGroundwaterLevelPoints,
    gwPoints => {
        const allKinds = gwPoints.map(point => {
            return {
                classes: point.classes,
                label: point.label,
                radius: point.radius
            };
        });
        return uniqWith(allKinds, isEqual);
    }
);

/*
 * Selector function which returns a function that returns an array of gw data appropriate
 * for use in a table.
 * @return {Function} - Function returns an array of visible ground water values with properties:
 *      @prop {String} parameterName
 *      @prop {String} result
 *      @prop {String} resultAccuracy
 *      @prop {Number} epochTime - useful for sorting the data by time
 *      @prop {String} dateTime in site's time zone.
 *      @prop {String} approvals
 *      @prop {String} qualifiers
 */
export const getGroundwaterLevelsTableData = createSelector(
    getSelectedGroundwaterLevels,
    gwLevels => {
        if (!gwLevels) {
            return [];
        }
        return gwLevels.values.map(value => {
            return {
                parameterName: gwLevels.parameter.name,
                result: value.result ? value.result : 'None',
                resultAccuracy: value.resultAccuracy ? value.resultAccuracy : 'None',
                epochTime: value.epochTime,
                dateTime: DateTime.fromMillis(value.epochTime, {zone: config.locationTimeZone})
                    .toFormat(TIME_FORMAT[value.phenomenonTimeAccuracy]),
                approvals: value.approvals.join(', '),
                qualifiers: value.qualifiers.join(', ')
            };
        });
    }
);
