import config from 'ui/config';


import {getGroundwaterLevelPoints, getGroundwaterLevelDataRange, getUniqueGWKinds,
    getGroundwaterLevelsTableData
} from './discrete-data';

describe('monitoring-location/components/hydrograph/selectors/discrete-data', () => {
    config.locationTimeZone = 'America/Chicago';

    const TEST_GW_LEVELS = [{
        USGSParameterCode: '72019',
        observedPropertyName: 'Depth to water level',
        observations: [
            {
                result: null,
                resultAccuracy: '0.1',
                approvals: ['Approved'],
                qualifiers: ['Static'],
                phenomenonTime: '2019-11-01T17',
                phenomenonTimeAccuracy: 'hour',
                epochTime: 1572627600000
            }, {
                result: '26.9',
                resultAccuracy: '0.1',
                approvals: ['Provisional'],
                qualifiers: ['Static'],
                phenomenonTime: '2020-01-29',
                phenomenonTimeAccuracy: 'day',
                epochTime: 1580277600000
            }, {
                result: '26.5',
                resultAccuracy: '0.5',
                approvals: ['Revised'],
                qualifiers: ['Static'],
                phenomenonTime: '2021',
                phenomenonTimeAccuracy: 'year',
                epochTime: 1609480800000
            }, {
                result: null,
                resultAccuracy: '0.1',
                approvals: ['Approved'],
                qualifiers: ['Dry', 'NoMeasurement'],
                phenomenonTime: '2021-04-23T16:34',
                phenomenonTimeAccuracy: 'minute',
                epochTime: 1619195640000
            }, {
                result: '26.1',
                resultAccuracy: '0.1',
                approvals: ['Approved'],
                qualifiers: ['Static'],
                phenomenonTime: '2021-08',
                phenomenonTimeAccuracy: 'month',
                epochTime: 1627794000000
            }
        ]
    }];
    const TEST_STATE = {
        hydrographData: {
            currentTimeRange: {
                start: 1484648660000,
                end: 1698303040000
            }
        },
        groundwaterLevelData: {
            all: TEST_GW_LEVELS
        },
        hydrographState: {
            selectedParameterCode: '72019'
        }
    };

    describe('getGroundwaterLevelPoints', () => {
        it('Return empty array if no groundwater levels are defined', () => {
            expect(getGroundwaterLevelPoints({
                ...TEST_STATE,
                groundwaterLevelData: {
                    all: []
                }
            })).toHaveLength(0);
        });

        it('Return the ground water points when groundwater levels are defined', () => {
            const points = getGroundwaterLevelPoints(TEST_STATE);
            expect(points).toHaveLength(3);

            expect(points[0].value).toEqual(26.9);
            expect(points[0].displayValue).toEqual('26.9');
            expect(points[0].dateTime).toEqual(1580277600000);
            expect(points[0].classes).toContain('provisional');
            expect(points[0].label).toEqual('Provisional');
            expect(points[0].radius).toBeDefined();

            expect(points[1].value).toEqual(26.5);
            expect(points[1].displayValue).toEqual('26.5');
            expect(points[1].dateTime).toEqual(1609480800000);
            expect(points[1].classes).toContain('revised');
            expect(points[1].label).toEqual('Revised');
            expect(points[1].radius).toBeDefined();

            expect(points[2].value).toEqual(26.1);
            expect(points[2].displayValue).toEqual('26.1');
            expect(points[2].dateTime).toEqual(1627794000000);
            expect(points[2].classes).toContain('approved');
            expect(points[2].label).toEqual('Approved');
            expect(points[2].radius).toBeDefined();
        });
    });

    describe('getGroundwaterLevelDataRange', () => {
        it('Return null if no ground water levels data', () => {
            expect(getGroundwaterLevelDataRange({
                ...TEST_STATE,
                groundwaterLevelData: {
                    all: []
                }
            })).toBeNull();
        });

        it('Return groundwater range if there is data', () => {
            expect(getGroundwaterLevelDataRange(TEST_STATE)).toEqual([26.1, 26.9]);
        });
    });

    describe('getUniqueGWKinds', () => {
        it('Return empty array if no groundwater levels are defined', () => {
            expect(getUniqueGWKinds({
                ...TEST_STATE,
                groundwaterLevelData: {
                    all: []
                }
            })).toHaveLength(0);
        });

        it('Return the unique kinds when groundwater levels are defined', () => {
            const points = getUniqueGWKinds(TEST_STATE);

            expect(points).toHaveLength(3);
            expect(points[0].classes).toContain('provisional');
            expect(points[0].label).toContain('Provisional');
            expect(points[0].radius).toBeDefined();

            expect(points[1].classes).toContain('revised');
            expect(points[1].label).toContain('Revised');
            expect(points[1].radius).toBeDefined();

            expect(points[2].classes).toContain('approved');
            expect(points[2].label).toContain('Approved');
            expect(points[2].radius).toBeDefined();
        });
    });


    describe('getGroundwaterLevelsTableData', () => {
        it('Returns an empty array if no groundwater levels', () => {
            expect(getGroundwaterLevelsTableData({
                ...TEST_STATE,
                groundwaterLevelData: {
                    all: []
                }
            })).toHaveLength(0);
        });

        it('Returns the array of groundwater levels table data', () => {
            const tableData = getGroundwaterLevelsTableData(TEST_STATE);
            expect(tableData).toHaveLength(5);
            expect(tableData[0]).toEqual({
                parameterName: 'Depth to water level',
                result: 'None',
                resultAccuracy: '0.1',
                epochTime: 1572627600000,
                dateTime: '2019-11-01 12-05:00',
                approvals: 'Approved',
                qualifiers: 'Static'
            });
            expect(tableData[1]).toEqual({
                parameterName: 'Depth to water level',
                result: '26.9',
                resultAccuracy: '0.1',
                epochTime: 1580277600000,
                dateTime: '2020-01-29',
                approvals: 'Provisional',
                qualifiers: 'Static'
            });
            expect(tableData[2]).toEqual({
                parameterName: 'Depth to water level',
                result: '26.5',
                resultAccuracy: '0.5',
                epochTime: 1609480800000,
                dateTime: '2021',
                approvals: 'Revised',
                qualifiers: 'Static'
            });
            expect(tableData[3]).toEqual({
                parameterName: 'Depth to water level',
                result: 'None',
                resultAccuracy: '0.1',
                epochTime: 1619195640000,
                dateTime: '2021-04-23 11:34-05:00',
                approvals: 'Approved',
                qualifiers: 'Dry, NoMeasurement'
            });
            expect(tableData[4]).toEqual({
                parameterName: 'Depth to water level',
                result: '26.1',
                resultAccuracy: '0.1',
                epochTime: 1627794000000,
                dateTime: '2021-08',
                approvals: 'Approved',
                qualifiers: 'Static'
            });
        });
    });
});
