
import config from 'ui/config';
import {lineMarker, rectangleMarker, textOnlyMarker, circleMarker} from 'd3render/markers';

import {getLegendMarkerRows} from './legend-data';

describe('monitoring-location/components/hydrograph/selectors/legend-data', () => {
    config.ivPeriodOfRecord = {
        '72019': {},
        '00060': {},
        '00010': {}
    };
    config.gwPeriodOfRecord = {
        '72019': {}
    };
    const TEST_PRIMARY_IV_DATA = {
        parameter: {
            parameterCode: '72019',
            unit: 'ft'
        },
        values: {
            '90649': {
                points: [
                    {value: '12.6', qualifiers: ['P'], dateTime: 1582560900000},
                    {value: null, qualifiers: ['ICE'], dateTime: 1582561800000}
                ],
                method: {
                    methodID: '90649',
                    methodDescription: 'from multi parameter sonde'
                }
            }
        }
    };

    const TEST_SECONDARY_IV_DATA = {
        parameter: {
            parameterCode: '00060',
                name: 'Streamflow, ft&#179;/s',
                description: 'Discharge, cubic feet per second',
                unit: 'ft3/s'
        },
        values: {
            57466: {
                points: [
                    {value: '12.6', qualifiers: ['P'], dateTime: 1582560900000},
                    {value: '12.6', qualifiers: ['A'], dateTime: 15825601800002},
                    {value: null, qualifiers: ['ICE'], dateTime: 1582561800000}
                ],
                    method: {
                    methodDescription: '',
                    methodID: '57466'
                }
            }
        }
    };
    const TEST_COMPARE_IV_DATA = {
        parameter: {
            parameterCode: '72019',
            unit: 'ft'
        },
        values: {
            '90649': {
                points: [
                    {value: '12.3', qualifiers: ['A'], dateTime: 1582560900000},
                    {value: '14.0', qualifiers: ['E'], dateTime: 1582561800000}
                ],
                method: {
                    methodID: '90649',
                    methodDescription: 'from multi parameter sonde'
                }
            }
        }
    };

    const TEST_GROUNDWATER_LEVELS = {
        USGSParameterCode: '72019',
        observations: [
            {result: '11.3', approvals: ['Revised'], qualifiers: ['Static'], epochTime: 1582560900000},
            {result: '12.2', approvals: ['Approved'], qualifiers: ['Static'], epochTime: 1582561800000}
        ]
    };

    const TEST_STATE = {
        hydrographData: {
            currentTimeRange: {
                start: 1582560800009,
                end: 1582561800001
            },
            thresholds: {
                operatingLimits: [{
                    parameterCode: '72019',
                    methodDescription: 'from multi parameter sonde',
                    thresholdDetails: [{
                        name: 'Very high',
                        referenceCode: 'Operational limit - high-Public',
                        referenceValue: '10'
                    }, {
                        name: 'Very low',
                        referenceCode: 'Operational limit - low-Public',
                        referenceValue: '1'
                    }]
                }]
            },
            prioryearTimeRange: {
                start: 1582560900000,
                end: 1582561800000
            },
            primaryIVData: TEST_PRIMARY_IV_DATA,
            compareIVData: TEST_COMPARE_IV_DATA
        },
        groundwaterLevelData: {
            all: [TEST_GROUNDWATER_LEVELS]
        },
        hydrographState: {
            selectedParameterCode: '72019',
            showCompareIVData: false,
            showMedianData: false,
            selectedIVMethodID: '90649'
        },
        floodData: {},
        ui: {
            windowWidth: 591,
            width: 542
        }
    };

    const TEST_STATE_TEMPERATURE = {
        ...TEST_STATE,
        hydrographState: {
            selectedParameterCode: '00010F',
            showCompareIVData: false,
            showMedianData: false,
            selectedIVMethodID: '90649'
        },
        primaryIVData: {
            ...TEST_PRIMARY_IV_DATA,
            parameter: {
                parameterCode: '00010F',
                unit: 'F'
            }
        }
    };

    describe('getLegendMarkerRows', () => {
        beforeAll(() => {
            Object.defineProperty(window, 'matchMedia', {
                writable: true,
                value: jest.fn().mockImplementation(query => ({
                    matches: false,
                    media: query,
                    onchange: null,
                    addEventListener: jest.fn(),
                    removeEventListener: jest.fn(),
                    dispatchEvent: jest.fn()
                }))
            });
        });
        it('Should return no data to show', () => {
            expect(getLegendMarkerRows({
                ...TEST_STATE,
                hydrographData: {},
                groundwaterLevelData: {
                    all: []
                }
            })).toHaveLength(0);
        });

        it('Should return markers for primary IV and gw Data', () => {
            const markerRows = getLegendMarkerRows(TEST_STATE);
            expect(markerRows).toHaveLength(2);
            const currentRow = markerRows[0];
            expect(currentRow).toHaveLength(3);
            expect(currentRow[0].type).toEqual(textOnlyMarker);
            expect(currentRow[1].type).toEqual(lineMarker);
            expect(currentRow[2].type).toEqual(rectangleMarker);

            const gwRow = markerRows[1];
            expect(gwRow).toHaveLength(3);
            expect(gwRow[0].type).toEqual(textOnlyMarker);
            expect(gwRow[1].type).toEqual(circleMarker);
            expect(gwRow[2].type).toEqual(circleMarker);
        });

        it('Should return markers for primary IV with a caculated temperature parameter code', () => {
            const markerRows = getLegendMarkerRows(TEST_STATE_TEMPERATURE);
            expect(markerRows).toHaveLength(2);
            const currentRow = markerRows[0];
            expect(currentRow).toHaveLength(3);
            expect(currentRow[0].type).toEqual(textOnlyMarker);
            expect(currentRow[1].type).toEqual(lineMarker);
            expect(currentRow[2].type).toEqual(rectangleMarker);
        });

        it('Should return markers for primary and compare when compare is visible', () => {
            const markerRows = getLegendMarkerRows({
                ...TEST_STATE,
                hydrographState: {
                    ...TEST_STATE.hydrographState,
                    showCompareIVData: true
                }
            });
            expect(markerRows).toHaveLength(3);
            const compareRow = markerRows[2];
            expect(compareRow).toHaveLength(3);
            expect(compareRow[0].type).toEqual(textOnlyMarker);
            expect(compareRow[1].type).toEqual(lineMarker);
            expect(compareRow[2].type).toEqual(lineMarker);
        });

        it('Should return a no data marker for compare if compare data has no points', () => {
            const markerRows = getLegendMarkerRows({
                ...TEST_STATE,
                hydrographData: {
                    ...TEST_STATE.hydrographData,
                    compareIVData: {
                        parameter: {
                            parameterCode: '72019'
                        },
                        values: {}
                    }
                },
                hydrographState: {
                    ...TEST_STATE.hydrographState,
                    showCompareIVData: true
                }
            });

            expect(markerRows).toHaveLength(3);
            const compareRow = markerRows[2];
            expect(compareRow).toHaveLength(2);
            expect(compareRow[0].type).toEqual(textOnlyMarker);
            expect(compareRow[1].type).toEqual(textOnlyMarker);
        });

        it('expects to show markers for secondary parameter data', () => {
            const markerRows = getLegendMarkerRows({
                ...TEST_STATE,
                hydrographData: {
                    primaryIVData: TEST_PRIMARY_IV_DATA,
                    secondaryIVData: TEST_SECONDARY_IV_DATA
                },
                groundwaterLevelData: {
                    all: []
                },
                dailyValueTimeSeriesData: {},
                hydrographState: {
                    ...TEST_STATE.hydrographState,
                    selectedSecondaryParameterCode: '00060',
                    selectedSecondaryIVMethodID: '57466'
                }
            });

            expect(markerRows).toHaveLength(3);
            expect(markerRows[0][0].text).toEqual('Current: ');
            expect(markerRows[0][1].text).toEqual('Provisional');
            expect(markerRows[0][2].text).toEqual('Ice Affected');
            expect(markerRows[1][0].text).toEqual('Secondary: ');
            expect(markerRows[1][1].text).toEqual('Provisional');
            expect(markerRows[1][2].text).toEqual('Approved');
            expect(markerRows[1][3].text).toEqual('Ice Affected');
            expect(markerRows[1][3].fill).toEqual('url(#hash-90)');
            expect(markerRows[2][0].text).toEqual('Field visit: ');
            expect(markerRows[2][1].text).toEqual('No data');
        });

        it('Should show markers for operational limits when limit is in graph range', () => {
            const TEST_PRIMARY_IV_DATA = {
                parameter: {
                    parameterCode: '00065',
                    unit: 'ft'
                },
                values: {
                    '69929': {
                        points: [
                            {value: '15', qualifiers: ['P'], dateTime: 1582560900001},
                            {value: '0', qualifiers: ['P'], dateTime: 1582561700000}
                        ],
                        method: {
                            methodID: '69929',
                            methodDescription: 'method 1'
                        }
                    }
                }
            };

            const TEST_STATE = {
                hydrographData: {
                    currentTimeRange: {
                        start: 1582560900000,
                        end: 1582561800000
                    },
                    thresholds: {
                        operatingLimits: [{
                            parameterCode: '00065',
                            methodDescription: 'method 1',
                            thresholdDetails: [{
                                name: 'Very high',
                                referenceCode: 'Operational limit - high-Public',
                                referenceValue: '10'
                            }, {
                                name: 'Very low',
                                referenceCode: 'Operational limit - low-Public',
                                referenceValue: '1'
                            }]
                        }]
                    },
                    primaryIVData: TEST_PRIMARY_IV_DATA
                },
                groundwaterLevelData: {
                    all: []
                },
                hydrographState: {
                    selectedIVMethodID: '69929'
                },
                floodData: {},
                ui: {
                    windowWidth: 591,
                    width: 542
                }
            };
            const markerRows = getLegendMarkerRows(TEST_STATE);

            expect(markerRows).toHaveLength(2);
            expect(markerRows[0][0].type).toEqual(textOnlyMarker);
            expect(markerRows[0][1].type).toEqual(lineMarker);
            expect(markerRows[0][0].text).toEqual('Maximum operating limit');
            expect(markerRows[0][1].text).toEqual('10 ft');
            expect(markerRows[1][0].type).toEqual(textOnlyMarker);
            expect(markerRows[1][1].type).toEqual(lineMarker);
            expect(markerRows[1][0].text).toEqual('Minimum operating limit');
            expect(markerRows[1][1].text).toEqual('1 ft');
        });

        it('Should NOT show markers for operational limits when limit is in NOT graph range', () => {
            const TEST_PRIMARY_IV_DATA = {
                parameter: {
                    parameterCode: '00065',
                    unit: 'ft'
                },
                values: {
                    '69929': {
                        points: [
                            {value: '15', qualifiers: ['P'], dateTime: 1582560900001},
                            {value: '0', qualifiers: ['P'], dateTime: 1582561700000}
                        ],
                        method: {
                            methodID: '69929',
                            methodDescription: 'method 1'
                        }
                    }
                }
            };

            const TEST_STATE = {
                hydrographData: {
                    currentTimeRange: {
                        start: 1582560900000,
                        end: 1582561800000
                    },
                    thresholds: {
                        operatingLimits: [{
                            parameterCode: '00065',
                            methodDescription: 'method 1',
                            thresholdDetails: [{
                                name: 'Very high',
                                referenceCode: 'Operational limit - high-Public',
                                referenceValue: '100'
                            }, {
                                name: 'Very low',
                                referenceCode: 'Operational limit - low-Public',
                                referenceValue: '1'
                            }]
                        }]
                    },
                    primaryIVData: TEST_PRIMARY_IV_DATA
                },
                groundwaterLevelData: {
                    all: []
                },
                hydrographState: {
                    selectedIVMethodID: '69929'
                },
                floodData: {},
                ui: {
                    windowWidth: 591,
                    width: 542
                }
            };
            const markerRows = getLegendMarkerRows(TEST_STATE);

            expect(markerRows).toHaveLength(1);
            expect(markerRows[0][0].type).toEqual(textOnlyMarker);
            expect(markerRows[0][1].type).toEqual(lineMarker);
            expect(markerRows[0][0].text).toEqual('Minimum operating limit');
            expect(markerRows[0][1].text).toEqual('1 ft');
        });
    });
});
