
import {getThresholdsInRange} from './thresholds-data';

describe('monitoring-location/components/hydrograph/selectors/thresholds-data', () => {
    const TEST_PRIMARY_IV_DATA = {
        parameter: {
            parameterCode: '00065',
            unit: 'ft'
        },
        values: {
            '69929': {
                points: [
                    {value: '15', qualifiers: ['P'], dateTime: 1582560900001},
                    {value: '0', qualifiers: ['P'], dateTime: 1582561700000}
                ],
                method: {
                    methodID: '69929',
                    methodDescription: 'method 1'
                }
            }
        }
    };

    const TEST_STATE = {
        hydrographData: {
            currentTimeRange: {
                start: 1582560900000,
                end: 1582561800000
            },
            thresholds: {
                operatingLimits: [{
                    parameterCode: '00065',
                    methodDescription: 'method 1',
                    thresholdDetails: [{
                        name: 'Very high',
                        referenceCode: 'Operational limit - high-Public',
                        referenceValue: '10'
                    }, {
                        name: 'Very low',
                        referenceCode: 'Operational limit - low-Public',
                        referenceValue: '1'
                    }]
                }]
            },
            primaryIVData: TEST_PRIMARY_IV_DATA
        },
        groundwaterLevelData: {
            all: []
        },
        hydrographState: {
            selectedIVMethodID: '69929'
        },
        ui: {
            windowWidth: 591,
            width: 542
        }
    };

    describe('getThresholdsInRange', () => {
        beforeAll(() => {
            Object.defineProperty(window, 'matchMedia', {
                writable: true,
                value: jest.fn().mockImplementation(query => ({
                    matches: false,
                    media: query,
                    onchange: null,
                    addEventListener: jest.fn(),
                    removeEventListener: jest.fn(),
                    dispatchEvent: jest.fn()
                }))
            });
        });
        it('Returns an empty array if there is no hydrograph data', () => {
            expect(getThresholdsInRange({
                ...TEST_STATE,
                hydrographData: {}
            })).toHaveLength(0);
        });

        it('Returns thresholds with correct values when there are thresholds in range', () => {
            const thresholds = getThresholdsInRange({
                ...TEST_STATE
            });
            expect(thresholds).toHaveLength(2);

            expect(thresholds[0].code).toBe('Operational limit - high-Public');
            expect(thresholds[0].thresholdClass).toBe('limit-high-public');
            expect(thresholds[0].unit).toBe('ft');
            expect(thresholds[0].value).toBe('10');

            expect(thresholds[1].code).toBe('Operational limit - low-Public');
            expect(thresholds[1].thresholdClass).toBe('limit-low-public');
            expect(thresholds[1].unit).toBe('ft');
            expect(thresholds[1].value).toBe('1');
        });

        it('Will not return high limit that is outside of range', () => {
            const thresholds = getThresholdsInRange({
                ...TEST_STATE,
                hydrographData: {
                    currentTimeRange: {
                        start: 1582560900000,
                        end: 1582561800000
                    },
                    thresholds: {
                        operatingLimits: [{
                            parameterCode: '00065',
                            methodDescription: 'method 1',
                            thresholdDetails: [{
                                name: 'Very high',
                                referenceCode: 'Operational limit - high-Public',
                                referenceValue: '22'
                            }, {
                                name: 'Very low',
                                referenceCode: 'Operational limit - low-Public',
                                referenceValue: '1'
                            }]

                        }]
                    },
                    primaryIVData: TEST_PRIMARY_IV_DATA
                }
            });
            expect(thresholds).toHaveLength(1);

            expect(thresholds[0].code).toBe('Operational limit - low-Public');
            expect(thresholds[0].thresholdClass).toBe('limit-low-public');
            expect(thresholds[0].unit).toBe('ft');
            expect(thresholds[0].value).toBe('1');
        });

        it('Will not return low limit that is outside of range', () => {
            const thresholds = getThresholdsInRange({
                ...TEST_STATE,
                hydrographData: {
                    currentTimeRange: {
                        start: 1582560900000,
                        end: 1582561800000
                    },
                    thresholds: {
                        operatingLimits: [{
                            parameterCode: '00065',
                            methodDescription: 'method 1',
                            thresholdDetails: [{
                                name: 'Very high',
                                referenceCode: 'Operational limit - high-Public',
                                referenceValue: '12'
                            }, {
                                name: 'Very low',
                                referenceCode: 'Operational limit - low-Public',
                                referenceValue: '-4'
                            }]

                        }]
                    },
                    primaryIVData: TEST_PRIMARY_IV_DATA
                }
            });
            expect(thresholds).toHaveLength(1);

            expect(thresholds[0].code).toBe('Operational limit - high-Public');
            expect(thresholds[0].thresholdClass).toBe('limit-high-public');
            expect(thresholds[0].unit).toBe('ft');
            expect(thresholds[0].value).toBe('12');
        });
    });
});
