import {select} from 'd3-selection';

import * as utils from 'ui/utils';

import {configureStore} from 'ml/store';
import {setMedianDataVisibility} from 'ml/store/hydrograph-state';

import {
    TEST_PRIMARY_IV_DATA, TEST_GW_LEVELS, TEST_STATS_DATA,
    TEST_CURRENT_TIME_RANGE
} from './mock-hydrograph-state';
import {drawTimeSeriesGraphData, initializeTimeSeriesGraph} from './time-series-graph';


const TEST_STATE = {
    hydrographData: {
        primaryIVData: TEST_PRIMARY_IV_DATA,
        statisticsData: TEST_STATS_DATA,
        currentTimeRange: TEST_CURRENT_TIME_RANGE
    },
    groundwaterLevelData: {
        all: [TEST_GW_LEVELS]
    },
    hydrographState: {
        selectedIVMethodID: '90649',
        showCompareIVData: false,
        showMedianData: false,
        graphCursorOffset: 300000,
        selectedParameterCode: '72019'
    },
    ui: {
        width: 400
    },
    floodData: {}
};

describe('monitoring-location/components/hydrograph/time-series-graph', () => {
    utils.mediaQuery = jest.fn().mockReturnValue(true);
    utils.wrap = jest.fn();

    let div;
    let store;

    beforeEach(() => {
        div = select('body').append('div').attr('id', 'hydrograph');
        store = configureStore(TEST_STATE);
    });

    afterEach(() => {
        div.remove();
    });

    it('Render graph title with monitoring location name in title and tooltip', () => {
        initializeTimeSeriesGraph(div, store, '11112222', 'USGS', 'This site', true, true);

        const titleDiv = div.select('.time-series-graph-title');
        expect(titleDiv.size()).toBe(1);
        expect(titleDiv.select('.monitoring-location-name-div').size()).toBe(1);
        expect(titleDiv.select('.usa-tooltip').size()).toBe(1);
    });

    it('Render graph title without monitoring location name in title', () => {
        initializeTimeSeriesGraph(div, store, '11112222', 'USGS', 'This site', false, true);

        const titleDiv = div.select('.time-series-graph-title');
        expect(titleDiv.size()).toBe(1);
        expect(titleDiv.select('.monitoring-location-name-div').size()).toBe(0);
        expect(titleDiv.select('.usa-tooltip').size()).toBe(1);
    });

    it('Render graph title without info toolip', () => {
        initializeTimeSeriesGraph(div, store, '11112222', 'USGS', 'This site', true, false);

        const titleDiv = div.select('.time-series-graph-title');
        expect(titleDiv.size()).toBe(1);
        expect(titleDiv.select('.monitoring-location-name-div').size()).toBe(1);
        expect(titleDiv.select('.usa-tooltip').size()).toBe(0);
    });

    it('Should render tooltip text and focus circles', () => {
        initializeTimeSeriesGraph(div, store, '11112222', 'USGS', 'This site', false, true);
        drawTimeSeriesGraphData(div, store, true);

        expect(div.selectAll('.tooltip-text-group').size()).toBe(1);
        expect(div.selectAll('.focus-line-group').size()).toBe(1);
        expect(div.selectAll('.focus-circle').size()).toBe(2);
    });

    it('Should not render tooltip text and focus circles', ()=> {
        initializeTimeSeriesGraph(div, store, '11112222', 'USGS', 'This site', false, false);
        drawTimeSeriesGraphData(div, store, false);

        expect(div.selectAll('.tooltip-text-group').size()).toBe(0);
        expect(div.selectAll('.focus-line-group').size()).toBe(0);
        expect(div.selectAll('.focus-circle').size()).toBe(0);
    });

    it('Should render current IV Data and groundwater levels but not median steps', () => {
        initializeTimeSeriesGraph(div, store, '11112222', 'USGS', 'This site', false, false);
        drawTimeSeriesGraphData(div, store, false);

        expect(div.selectAll('.ts-primary-group').html()).not.toEqual('');
        expect(div.selectAll('.ts-compare-group').html()).toEqual('');
        expect(div.selectAll('.iv-graph-gw-levels-group').html()).not.toEqual('');
        expect(div.selectAll('.median-stats-group').size()).toBe(0);
    });

    it('Should render median data if visible', () => {
        store.dispatch(setMedianDataVisibility(true));
        initializeTimeSeriesGraph(div, store, '11112222', 'USGS', 'This site', false, false);
        drawTimeSeriesGraphData(div, store, false);

        expect(div.selectAll('.ts-primary-group').html()).not.toEqual('');
        expect(div.selectAll('.ts-compare-group').html()).toEqual('');
        expect(div.selectAll('.iv-graph-gw-levels-group').html()).not.toEqual('');
        expect(div.selectAll('.median-stats-group').size()).toBe(2);
    });

    it('expects the correct tooltip text when the description is available', () => {
        initializeTimeSeriesGraph(div, store, '11112222', 'USGS', 'This site', true, true);

        const titleDiv = div.select('.time-series-graph-title');
        expect(titleDiv.select('.usa-tooltip').text()).toBe('Depth to water level, feet');
    });

    it('expects the correct tooltip text when the description is NOT available', () => {
       const NO_PARAMETER_DATA =  {
           parameter: {
               parameterCode: '00065'
           },
           values: {}
       };

        store = configureStore({
            ...TEST_STATE,
            hydrographData: {
                primaryIVData: NO_PARAMETER_DATA
            }
        });

        initializeTimeSeriesGraph(div, store, '11112222', 'USGS', 'This site', true, true);

        const titleDiv = div.select('.time-series-graph-title');
        expect(titleDiv.select('.usa-tooltip').text()).toBe('No description available');
    });
});
