import {select} from 'd3-selection';

import config from 'ui/config';
import * as utils from 'ui/utils';

import {configureStore} from 'ml/store';

import {TEST_GW_LEVELS, TEST_PRIMARY_IV_DATA, TEST_SECONDARY_IV_DATA, TEST_CURRENT_TIME_RANGE} from './mock-hydrograph-state';
import {drawTooltipText, drawTooltipFocus} from './tooltip';

describe('monitoring-location/components/hydrograph/tooltip module', () => {
    utils.mediaQuery = jest.fn().mockReturnValue(true);
    config.locationTimeZone = 'America/Chicago';
    const TEST_STATE = {
        hydrographData: {
            primaryIVData: TEST_PRIMARY_IV_DATA,
            currentTimeRange: TEST_CURRENT_TIME_RANGE
        },
        groundwaterLevelData: {
            all: [TEST_GW_LEVELS]
        },
        hydrographState: {
            selectedIVMethodID: '90649',
            graphCursorOffset: 500000,
            selectedParameterCode: '72019'
        },
        ui: {
            windowWidth: 1300,
            width: 990
        }
    };

    describe('drawTooltipText', () => {
        let div;
        let store;
        beforeEach(() => {
            div = select('body').append('div');
            store = configureStore(TEST_STATE);
        });

        afterEach(() => {
            div.remove();
        });

        it('Creates the container for tooltips', () => {
            div.call(drawTooltipText, store);

            const textGroup = div.selectAll('.tooltip-text-group');
            expect(textGroup.size()).toBe(1);
        });

        it('Creates the text elements with the label for the focus times', () => {
            div.call(drawTooltipText, store);
            let value = div.select('.primary-tooltip-text').text().split(' - ')[0];
            expect(value).toBe('24.1 ft');

            expect(div.select('.compare-tooltip-text').size()).toBe(0);

            value = div.select('.gw-level-point').text().split(' - ')[0];
            expect(value).toBe('27.2 ft');
        });

        it('expects that if a secondary parameter is graphed that it will have a tool tip', () => {
            const TEST_STATE_SECONDARY_PARAMETER = {
                hydrographData: {
                    primaryIVData: TEST_PRIMARY_IV_DATA,
                    secondaryIVData: TEST_SECONDARY_IV_DATA,
                    currentTimeRange: TEST_CURRENT_TIME_RANGE
                },
                groundwaterLevelData: {
                    all: [TEST_GW_LEVELS]
                },
                hydrographState: {
                    selectedIVMethodID: '90649',
                    graphCursorOffset: 500000,
                    selectedParameterCode: '72019',
                    selectedSecondaryParameterCode: '00065'
                },
                ui: {
                    windowWidth: 1300,
                    width: 990
                }
            };

            store = configureStore(TEST_STATE_SECONDARY_PARAMETER);

            div.call(drawTooltipText, store);
            expect(div.select('.primary-tooltip-text').size()).toBe(1);
            expect(div.select('.primary-tooltip-text').text()).toBe('24.1 ft - Feb 24, 2020 10:30:00 AM CST');

            expect(div.select('.secondary-tooltip-text').size()).toBe(1);
            expect(div.select('.secondary-tooltip-text').text()).toBe('3.2 ft - Feb 24, 2020 10:30:00 AM CST');
        });
    });

    describe('createTooltipFocus', () => {
        let store;
        let svg;
        beforeEach(() => {
            svg = select('body').append('svg');
        });

        afterEach(() => {
            svg.remove();
        });

        it('Creates focus lines and focus circles when cursor not set', () => {
            store = configureStore(TEST_STATE);

            svg.call(drawTooltipFocus, store);

            expect(svg.selectAll('.focus-line').size()).toBe(1);
            expect(svg.selectAll('.focus-circle').size()).toBe(2);
            expect(svg.select('.focus-overlay').size()).toBe(1);
        });


        it('expects that if a secondary parameter is graphed that it will have a focus dot', () => {
            const TEST_STATE_SECONDARY_PARAMETER = {
                hydrographData: {
                    primaryIVData: TEST_PRIMARY_IV_DATA,
                    secondaryIVData: TEST_SECONDARY_IV_DATA,
                    currentTimeRange: TEST_CURRENT_TIME_RANGE
                },
                groundwaterLevelData: {
                    all: [TEST_GW_LEVELS]
                },
                hydrographState: {
                    selectedIVMethodID: '90649',
                    graphCursorOffset: 500000,
                    selectedParameterCode: '72019',
                    selectedSecondaryParameterCode: '00065'
                },
                ui: {
                    windowWidth: 1300,
                    width: 990
                }
            };

            store = configureStore(TEST_STATE_SECONDARY_PARAMETER);

            svg.call(drawTooltipFocus, store);

            expect(svg.selectAll('.focus-line').size()).toBe(1);
            expect(svg.selectAll('.focus-circle').size()).toBe(3);
            expect(svg.select('.focus-overlay').size()).toBe(1);
        });
    });
});
