import {bindActionCreators} from 'redux';
import ReduxConnectVue from 'redux-connect-vue';
import {createStructuredSelector} from 'reselect';
import {mount} from '@vue/test-utils';

import config from 'ui/config';

import {configureStore} from 'ml/store';

import {TEST_PRIMARY_IV_DATA, TEST_GW_LEVELS, TEST_SECONDARY_IV_DATA} from '../mock-hydrograph-state';

import DataTable from './data-table.vue';
import DownloadData from './download-data.vue';

describe('monitoring-location/components/hydrograph/components/data-table.vue', () => {
    config.locationTimeZone = 'America/Chicago';
    const TEST_STATE = {
        hydrographData: {
            currentTimeRange: {
                start: 1582560000000,
                end: 1600620000000
            },
            primaryIVData: TEST_PRIMARY_IV_DATA
        },
        groundwaterLevelData: {
            all: [TEST_GW_LEVELS]
        },
        hydrographState: {
            selectedParameterCode: '72019',
            selectedIVMethodID: '90649'
        }
    };

    const TEST_STATE_ALL_TABLES = {
        hydrographData: {
            currentTimeRange: {
                start: 1582560000000,
                end: 1600620000000
            },
            primaryIVData: TEST_PRIMARY_IV_DATA,
            secondaryIVData: TEST_SECONDARY_IV_DATA
        },
        groundwaterLevelData: {
            all: [TEST_GW_LEVELS]
        },
        hydrographState: {
            selectedParameterCode: '72019',
            selectedIVMethodID: '90649',
            selectedSecondaryIVMethodID: '57465',
            selectedSecondaryParameterCode: '00065'
        }
    };

    let store;
    let wrapper;

    it('Shows both the IV and GW data table', () => {
        store = configureStore(TEST_STATE);

        wrapper = mount(DataTable, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store
                }
            }
        });

        expect(wrapper.find('#iv-primary-table-container').isVisible()).toBe(true);
        expect(wrapper.find('#iv-secondary-table-container').isVisible()).toBe(false);
        expect(wrapper.find('#gw-table-container').isVisible()).toBe(true);
    });

    it('Expects that if there is no secondary table that the primary title is correct', () => {
        store = configureStore(TEST_STATE);

        wrapper = mount(DataTable, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store
                }
            }
        });

        expect(wrapper.find('#iv-primary-table-container').isVisible()).toBe(true);
        expect(wrapper.find('#iv-primary-table-container caption').text()).toBe('Depth to water level -- instantaneous value data');
        expect(wrapper.find('#iv-secondary-table-container').isVisible()).toBe(false);
        expect(wrapper.find('#gw-table-container').isVisible()).toBe(true);
    });

    it('Shows only the IV data table if there is no groundwater data', () => {
        store = configureStore({
            ...TEST_STATE,
            groundwaterLevelData: {
                all: []
            }
        });

        wrapper = mount(DataTable, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store
                }
            }
        });

        expect(wrapper.find('#iv-primary-table-container').isVisible()).toBe(true);
        expect(wrapper.find('#iv-secondary-table-container').isVisible()).toBe(false);
        expect(wrapper.find('#gw-table-container').isVisible()).toBe(false);
    });

    it('Shows only the GW data table if there is groundwater data and no IV data', () => {
        store = configureStore({
            ...TEST_STATE,
            hydrographData: {
                currentTimeRange: {
                    start: 1582560000000,
                    end: 1600620000000
                },
                primaryIVData: null
            },
            groundwaterLevelData: {
                all: [TEST_GW_LEVELS]
            }
        });

        wrapper = mount(DataTable, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store
                }
            }
        });

        expect(wrapper.find('#iv-primary-table-container').isVisible()).toBe(false);
        expect(wrapper.find('#iv-secondary-table-container').isVisible()).toBe(false);
        expect(wrapper.find('#gw-table-container').isVisible()).toBe(true);
    });

    it('Shows both IV primary and secondary parameter selection tables and GW data table if data for each is in the state', () => {
        store = configureStore(TEST_STATE_ALL_TABLES);

        wrapper = mount(DataTable, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store
                }
            }
        });

        expect(wrapper.find('#iv-primary-table-container').isVisible()).toBe(true);
        expect(wrapper.find('#iv-secondary-table-container').isVisible()).toBe(true);
        expect(wrapper.find('#gw-table-container').isVisible()).toBe(true);
    });

    it('Expects all tables will have the correct titles', () => {
        store = configureStore(TEST_STATE_ALL_TABLES);

        wrapper = mount(DataTable, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store
                }
            }
        });

        expect(wrapper.find('#iv-primary-table-container caption').text()).toBe('Depth to water level -- instantaneous value data, primary axis');
        expect(wrapper.find('#iv-secondary-table-container caption').text()).toBe('Gage height, ft -- instantaneous value data, secondary axis');
        expect(wrapper.find('#gw-table-container caption').text()).toBe('Depth to water level -- field visit data');
    });

    it('Shows only data retrieve button and download container', async() => {
        store = configureStore(TEST_STATE);

        wrapper = mount(DataTable, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store,
                    siteno: '11112222',
                    agencyCd: 'USGS',
                    buttonSetName: 'test-buttons'
                }
            }
        });

        expect(wrapper.findAll('button')).toHaveLength(1);
        expect(wrapper.find('button').text()).toBe('Retrieve data');
        expect(wrapper.findAllComponents(DownloadData)).toHaveLength(0);
        await wrapper.find('button').trigger('click');
        expect(wrapper.findAllComponents(DownloadData)).toHaveLength(1);
        await wrapper.find('button').trigger('click');
        expect(wrapper.findAllComponents(DownloadData)).toHaveLength(0);
    });
});
