import {mount} from '@vue/test-utils';
import ReduxConnectVue from 'redux-connect-vue';
import {createStructuredSelector} from 'reselect';

import {USWDSAlert} from '@wma/wdfn-vue-components';

import config from 'ui/config';

import {configureStore} from 'ml/store';
import {setCompareDataVisibility, setMedianDataVisibility} from 'ml/store/hydrograph-state';

import {
    TEST_CURRENT_TIME_RANGE,
    TEST_COMPARE_TIME_RANGE,
    TEST_PRIMARY_IV_DATA,
    TEST_STATS_DATA,
    TEST_GW_LEVELS,
    TEST_SECONDARY_IV_DATA
} from '../mock-hydrograph-state';

import DownloadData from './download-data.vue';

describe('monitoring-location/components/hydrograph/vue-components/download-data', () => {
    config.SITE_DATA_ENDPOINT = 'https://fakeserviceroot.com/nwis/site';
    config.IV_DATA_ENDPOINT = 'https://fakeserviceroot.com/nwis/iv';
    config.HISTORICAL_IV_DATA_ENDPOINT = 'https://fakeserviceroot-more-than-120-days.com/nwis/iv';
    config.STATISTICS_ENDPOINT = 'https://fakeserviceroot.com/nwis/stat';
    config.GROUNDWATER_LEVELS_ENDPOINT = 'https://fakegroundwater.org/gwlevels/';
    config.locationTimeZone = 'America/Chicago';

    const TEST_STATE = {
        hydrographData: {
            currentTimeRange: TEST_CURRENT_TIME_RANGE,
            prioryearTimeRange: TEST_COMPARE_TIME_RANGE,
            primaryIVData: TEST_PRIMARY_IV_DATA,
            secondaryIVData: TEST_SECONDARY_IV_DATA,
            compareIVData: TEST_PRIMARY_IV_DATA,
            statisticsData: TEST_STATS_DATA
        },
        groundwaterLevelData: {
            all: [TEST_GW_LEVELS]
        },
        hydrographState: {
            showCompareIVData: false,
            showMedianData: false,
            selectedIVMethodID: '90649',
            selectedParameterCode: '72019',
            selectedSecondaryParameterCode: '00065',
            selectedSecondaryIVMethodID: '57465'
        }
    };

    describe('tests for download data component', () => {
        let store;
        let windowSpy;
        let wrapper;

        beforeEach(() => {
            windowSpy = jest.spyOn(window, 'open').mockImplementation(() => null);
            store = configureStore(TEST_STATE);
            wrapper = mount(DownloadData, {
                props: {
                    buttonSetName: 'test-buttons'
                },
                global: {
                    plugins: [
                        [ReduxConnectVue, {
                            store,
                            mapStateToPropsFactory: createStructuredSelector
                        }]
                    ],
                    provide: {
                        store: store,
                        siteno: '11112222',
                        agencyCd: 'USGS'
                    }
                }
            });
        });

        it('Renders form with the appropriate radio buttons and download button', () => {
            expect(wrapper.findAll('input[type="radio"]')).toHaveLength(4);
            expect(wrapper.findAll('#test-buttons-primary-data-download-button')).toHaveLength(1);
            expect(wrapper.findAll('#test-buttons-groundwater-levels-data-download-button')).toHaveLength(1);
            expect(wrapper.findAll('#test-buttons-secondary-data-download-button')).toHaveLength(1);
            expect(wrapper.findAll('#test-buttons-site-data-download-button')).toHaveLength(1);
        });

        it('Rerenders the radio buttons if data visibility changes', async() => {
            expect(wrapper.findAll('input[type="radio"]')).toHaveLength(4);
            await store.dispatch(setCompareDataVisibility(true));
            expect(wrapper.findAll('input[type="radio"]')).toHaveLength(5);
            await store.dispatch(setMedianDataVisibility(true));  
            expect(wrapper.findAll('input[type="radio"]')).toHaveLength(6);
        });

        it('Shows an error message if the download button is clicked with no radio buttons checked', async() => {
            const downloadButton = wrapper.find('button.download-selected-data');
            await downloadButton.trigger('click');
            expect(wrapper.findAllComponents(USWDSAlert)).toHaveLength(1);
        });

        it('Opens a window with the URL for the selected data', async() => {
            const downloadButton = wrapper.find('button.download-selected-data');
            const siteDataButton = wrapper.find('#test-buttons-site-data-download-button');
            await siteDataButton.trigger('click');
            await downloadButton.trigger('click');

            expect(wrapper.findAllComponents(USWDSAlert)).toHaveLength(0);
            expect(windowSpy.mock.calls).toHaveLength(1);
            expect(windowSpy.mock.calls[0][0]).toContain('/site/');
            expect(windowSpy.mock.calls[0][0]).toContain('sites=11112222');
        });

        it('Opens window with correct data for data type selected and downloaded', async() => {
            const downloadButton = wrapper.find('button.download-selected-data');
            await store.dispatch(setMedianDataVisibility(true));

            const primaryDataButton = wrapper.find('#test-buttons-primary-data-download-button');
            await primaryDataButton.trigger('click');
            await downloadButton.trigger('click');
            expect(windowSpy.mock.calls[0][0]).toContain('/iv/');
            expect(windowSpy.mock.calls[0][0]).toContain('sites=11112222');
            expect(windowSpy.mock.calls[0][0]).toContain('parameterCd=72019');
            expect(windowSpy.mock.calls[0][0]).toContain('startDT=2020-02-24T10:15:00.000-06:00');
            expect(windowSpy.mock.calls[0][0]).toContain('endDT=2020-09-20T11:45:00.000-05:00');


            const medianDataButton = wrapper.find('#test-buttons-median-data-download-button');
            await medianDataButton.trigger('click');
            await downloadButton.trigger('click');
            expect(windowSpy.mock.calls[1][0]).toContain('/stat/');
            expect(windowSpy.mock.calls[1][0]).toContain('statTypeCd=median');
            expect(windowSpy.mock.calls[1][0]).toContain('parameterCd=72019');


            const groundWaterButton = wrapper.find('#test-buttons-groundwater-levels-data-download-button');
            await groundWaterButton.trigger('click');
            await downloadButton.trigger('click');
            expect(windowSpy.mock.calls[2][0]).toContain('/gwlevels/');
            expect(windowSpy.mock.calls[2][0]).toContain('featureId=USGS-11112222');
        });

        it('Opens window for compare data when selected and downloaded', async() => {
            const downloadButton = wrapper.find('button.download-selected-data');
            await store.dispatch(setCompareDataVisibility(true));

            const compareDataButton = wrapper.find('#test-buttons-compare-data-download-button');
            await compareDataButton.trigger('click');
            await downloadButton.trigger('click');
            expect(windowSpy.mock.calls[0][0]).toContain('/iv/');
            expect(windowSpy.mock.calls[0][0]).toContain('sites=11112222');
            expect(windowSpy.mock.calls[0][0]).toContain('parameterCd=72019');
            expect(windowSpy.mock.calls[0][0]).toContain('startDT=2012-06-13T15:57:44.000-05:00');
            expect(windowSpy.mock.calls[0][0]).toContain('endDT=2013-06-13T15:57:44.000-05:00');
        });

        it('Opens window for secondary parameter data when selected and downloaded', async() => {
            const downloadButton = wrapper.find('button.download-selected-data');
            const secondaryDataButton = wrapper.find('#test-buttons-secondary-data-download-button');
            await secondaryDataButton.trigger('click');
            await downloadButton.trigger('click');
            expect(windowSpy.mock.calls[0][0]).toContain('/iv/');
            expect(windowSpy.mock.calls[0][0]).toContain('sites=11112222');
            expect(windowSpy.mock.calls[0][0]).toContain('parameterCd=00065');
            expect(windowSpy.mock.calls[0][0]).toContain('startDT=2020-02-24T10:15:00.000-06:00');
            expect(windowSpy.mock.calls[0][0]).toContain('endDT=2020-09-20T11:45:00.000-05:00');
        });

        it('Expects the error alert to disappear once a user selects a radio', async() => {
            const downloadButton = wrapper.find('button.download-selected-data');
            await downloadButton.trigger('click');
            expect(wrapper.findAllComponents(USWDSAlert)).toHaveLength(1);

            const siteDataButton = wrapper.find('#test-buttons-site-data-download-button');
            await siteDataButton.trigger('click');
            expect(wrapper.findAllComponents(USWDSAlert)).toHaveLength(0);
        });

        it('Expects that removing the secondary parameter from state will remove button', () => {
            store = configureStore({
                ...TEST_STATE,
                hydrographState: {
                    showCompareIVData: false,
                    showMedianData: false,
                    selectedIVMethodID: '90649',
                    selectedParameterCode: '72019',
                    selectedSecondaryParameterCode: null,
                    selectedSecondaryIVMethodID: null
                }
            });
            wrapper = mount(DownloadData, {
                props: {
                    buttonSetName: 'test-buttons'
                },
                global: {
                    plugins: [
                        [ReduxConnectVue, {
                            store,
                            mapStateToPropsFactory: createStructuredSelector
                        }]
                    ],
                    provide: {
                        store: store,
                        siteno: '11112222',
                        agencyCd: 'USGS'
                    }
                }
            });

            expect(wrapper.findAll('input[type="radio"]')).toHaveLength(3);
            expect(wrapper.findAll('#test-buttons-primary-data-download-button')).toHaveLength(1);
            expect(wrapper.findAll('#test-buttons-groundwater-levels-data-download-button')).toHaveLength(1);
            expect(wrapper.findAll('#test-buttons-secondary-data-download-button')).toHaveLength(0);
            expect(wrapper.findAll('#test-buttons-site-data-download-button')).toHaveLength(1);
        });

        it('expects the primary radio button will not show if the parameter is calculated--00010F', () => {
            store = configureStore({
                ...TEST_STATE,
                hydrographData: {
                    currentTimeRange: TEST_CURRENT_TIME_RANGE,
                    primaryIVData: {
                        ...TEST_PRIMARY_IV_DATA,
                        parameter: {
                            parameterCode: '00010F',
                            name: 'Calculated Temp',
                            description: 'F (Calculated)',
                            unit: 'F'
                        }
                    }
                },
                hydrographState: {
                    showCompareIVData: false,
                    showMedianData: false,
                    selectedIVMethodID: '90649',
                    selectedParameterCode: '00010F',
                    selectedSecondaryParameterCode: null,
                    selectedSecondaryIVMethodID: null
                }
            });
            wrapper = mount(DownloadData, {
                props: {
                    buttonSetName: 'test-buttons'
                },
                global: {
                    plugins: [
                        [ReduxConnectVue, {
                            store,
                            mapStateToPropsFactory: createStructuredSelector
                        }]
                    ],
                    provide: {
                        store: store,
                        siteno: '11112222',
                        agencyCd: 'USGS'
                    }
                }
            });

            expect(wrapper.findAll('input[type="radio"]')).toHaveLength(1);
            expect(wrapper.findAll('#test-buttons-primary-data-download-button')).toHaveLength(0);
            expect(wrapper.findAll('#test-buttons-site-data-download-button')).toHaveLength(1);
        });

        it('Expects that secondary parameter button will not show for calculated parameter-00010F', () => {
            store = configureStore({
                ...TEST_STATE,
                hydrographState: {
                    showCompareIVData: false,
                    showMedianData: false,
                    selectedIVMethodID: '90649',
                    selectedParameterCode: '72019',
                    selectedSecondaryParameterCode: '00010F',
                    selectedSecondaryIVMethodID: null
                }
            });
            wrapper = mount(DownloadData, {
                props: {
                    buttonSetName: 'test-buttons'
                },
                global: {
                    plugins: [
                        [ReduxConnectVue, {
                            store,
                            mapStateToPropsFactory: createStructuredSelector
                        }]
                    ],
                    provide: {
                        store: store,
                        siteno: '11112222',
                        agencyCd: 'USGS'
                    }
                }
            });

            expect(wrapper.findAll('input[type="radio"]')).toHaveLength(3);
            expect(wrapper.findAll('#test-buttons-primary-data-download-button')).toHaveLength(1);
            expect(wrapper.findAll('#test-buttons-groundwater-levels-data-download-button')).toHaveLength(1);
            expect(wrapper.findAll('#test-buttons-secondary-data-download-button')).toHaveLength(0);
            expect(wrapper.findAll('#test-buttons-site-data-download-button')).toHaveLength(1);
        });
    });
});
