import {bindActionCreators} from 'redux';
import ReduxConnectVue from 'redux-connect-vue';
import {createStructuredSelector} from 'reselect';
import {ref} from 'vue';
import {mount} from '@vue/test-utils';

import config from 'ui/config';
import * as utils from 'ui/utils';

import {configureStore} from 'ml/store';
import {Actions as uiActions} from 'ml/store/ui-state';

import {TEST_CURRENT_TIME_RANGE, TEST_PRIMARY_IV_DATA, TEST_GW_LEVELS} from '../mock-hydrograph-state';

import GraphBrush from './graph-brush.vue';

describe('monitoring-location/components/hydrograph/vue-components/graph-brush.vue', () => {
    utils.mediaQuery = jest.fn();

    config.ivPeriodOfRecord = {
        '72019': {},
        '00010': {}
    };

    const TEST_STATE = {
        hydrographData: {
            currentTimeRange: TEST_CURRENT_TIME_RANGE,
            primaryIVData: TEST_PRIMARY_IV_DATA
        },
        groundwaterLevelData: {
            all: [TEST_GW_LEVELS]
        },
        hydrographState: {
            selectedParameterCode: '72019',
            selectedIVMethodID: '90649'
        },
        ui: {
            windowWidth: 1000,
            width: 900
        }
    };

    let store;
    let wrapper;

    const initialLoadingComplete = ref(null);

    describe('With IV and GW data in the store', () => {
        beforeEach(() => {
            store = configureStore(TEST_STATE);

            initialLoadingComplete.value = false;
            utils.mediaQuery.mockReturnValue(true);
            wrapper = mount(GraphBrush, {
                global: {
                    plugins: [
                        [ReduxConnectVue, {
                            store,
                            mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                            mapStateToPropsFactory: createStructuredSelector
                        }]
                    ],
                    provide: {
                        store: store,
                        initialLoadingComplete: initialLoadingComplete
                    }
                }
            });
        });

        it('Expect that the graph brush is rendered but no data segments or points are rendered within', () => {
            expect(wrapper.find('.brush-svg').attributes('viewBox')).not.toBe('');
            expect(wrapper.find('.brush-text-hint').attributes('transform')).not.toBe('');
            expect(wrapper.find('.graph-brush-group').attributes('transform')).not.toBe('');
            expect(wrapper.find('.brush').exists()).toBe(true);
            expect(wrapper.find('.x-axis').exists()).toBe(true);
            expect(wrapper.find('.ts-primary-group').exists()).toBe(false);
            expect(wrapper.find('.iv-graph-gw-levels-group').exists()).toBe(false);
        });

        it('Expects that setting initialLoadingComplete to true renders the IV and GW levels data', async() => {
            initialLoadingComplete.value = true;
            await wrapper.vm.$nextTick();

            const ivDataGroup = wrapper.find('.ts-primary-group');
            expect(ivDataGroup.exists()).toBe(true);
            expect(ivDataGroup.find('path').exists()).toBe(true);
            const gwDataGroup = wrapper.find('.iv-graph-gw-levels-group');
            expect(gwDataGroup.exists()).toBe(true);
            expect(gwDataGroup.find('circle').exists()).toBe(true);
        });

        it('Expects that changing the ui width will change the viewBox and transforms', async() => {
            const viewBox = wrapper.find('.brush-svg').attributes('viewBox');
            const captionTransform = wrapper.find('.brush-text-hint').attributes('transform');
            const centerTransform = wrapper.find('.graph-brush-group').attributes('transform');
            utils.mediaQuery.mockReturnValue(false);
            store.dispatch(uiActions.resizeUI(400, 350));
            await wrapper.vm.$nextTick();

            expect(wrapper.find('.brush-svg').attributes('viewBox')).not.toEqual(viewBox);
            expect(wrapper.find('.brush-text-hint').attributes('transform')).not.toEqual(captionTransform);
            expect(wrapper.find('.graph-brush-group').attributes('transform')).not.toEqual(centerTransform);
        });
    });

    it('Expects that if the store only contains IV data that the gwDataGroup is empty', async() => {
        utils.mediaQuery.mockReturnValue(true);
        initialLoadingComplete.value = false;
        store = configureStore({
            ...TEST_STATE,
            groundwaterLevelData: {
                all: []
            }
        });
        wrapper = mount(GraphBrush, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store,
                    initialLoadingComplete: initialLoadingComplete
                }
            }
        });

        await wrapper.vm.$nextTick();
        initialLoadingComplete.value = true;
        await wrapper.vm.$nextTick();

        const ivDataGroup = wrapper.find('.ts-primary-group');
        expect(ivDataGroup.exists()).toBe(true);
        expect(ivDataGroup.find('path').exists()).toBe(true);
        const gwDataGroup = wrapper.find('.iv-graph-gw-levels-group');
        expect(gwDataGroup.exists()).toBe(true);
        expect(gwDataGroup.find('circle').exists()).toBe(false);
    });

    it('Expects that if the store only contains GW data that the IV data group is empty', async() => {
        utils.mediaQuery.mockReturnValue(true);
        initialLoadingComplete.value = false;
        store = configureStore({
            ...TEST_STATE,
            hydrographData: {
                ...TEST_STATE.hydrographData,
                primaryIVData: {
                    ...TEST_STATE.hydrographData.primaryIVData,
                    values: {}
                }
            }
        });
        wrapper = mount(GraphBrush, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store,
                    initialLoadingComplete: initialLoadingComplete
                }
            }
        });

        await wrapper.vm.$nextTick();
        initialLoadingComplete.value = true;
        await wrapper.vm.$nextTick();

        const ivDataGroup = wrapper.find('.ts-primary-group');
        expect(ivDataGroup.exists()).toBe(true);
        expect(ivDataGroup.find('path').exists()).toBe(false);
        const gwDataGroup = wrapper.find('.iv-graph-gw-levels-group');
        expect(gwDataGroup.exists()).toBe(true);
        expect(gwDataGroup.find('circle').exists()).toBe(true);
    });
});