import {enableFetchMocks, disableFetchMocks} from 'jest-fetch-mock';
import mockConsole from 'jest-mock-console';
import {bindActionCreators} from 'redux';
import ReduxConnectVue from 'redux-connect-vue';
import {createStructuredSelector} from 'reselect';
import {mount} from '@vue/test-utils';

import config from 'ui/config';
import * as utils from 'ui/utils';

import {configureStore} from 'ml/store';
import * as hydrographData from 'ml/store/hydrograph-data';
import {setSelectedTimeSpan} from 'ml/store/hydrograph-state';

import {TEST_CURRENT_TIME_RANGE} from '../mock-hydrograph-state';
import * as dataIndicator from '../data-indicator';

import GraphControls from './graph-controls.vue';

describe('monitoring-location/components/hydrograph/components/graph-controls', () => {
    utils.mediaQuery = jest.fn().mockReturnValue(true);
    config.ivPeriodOfRecord = {
        '72019': {},
        '00010': {}
    };

    let restoreConsole;
    let store;
    let retrievePriorYearSpy, retrieveStatisticsSpy;
    let showDataIndicatorSpy;
    let wrapper;

    beforeAll(() => {
        enableFetchMocks();
        restoreConsole = mockConsole();
    });

    afterAll(() => {
        disableFetchMocks();
        restoreConsole();
    });

    beforeEach(() => {
        retrievePriorYearSpy = jest.spyOn(hydrographData, 'retrievePriorYearIVData');
        retrieveStatisticsSpy = jest.spyOn(hydrographData, 'retrieveStatistics');
        showDataIndicatorSpy = jest.spyOn(dataIndicator, 'showDataIndicators');

        store = configureStore({
            hydrographData: {
                currentTimeRange: TEST_CURRENT_TIME_RANGE
            },
            hydrographState: {
                showCompareIVData: false,
                selectedTimeSpan: 'P7D',
                showMedianData: false,
                selectedParameterCode: '72019'
            }
        });

        wrapper = mount(GraphControls, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store,
                    siteno: '12345678'
                }
            }
        });
    });

    it('expect two checkboxes that are unchecked', () => {
        const checkboxes = wrapper.findAll('.usa-checkbox');
        expect(checkboxes).toHaveLength(2);
        const compareInput = checkboxes[0].find('input');
        expect(compareInput.attributes('value')).toBe('compare');
        expect(compareInput.attributes('checked')).not.toBeDefined();
        const medianInput = checkboxes[1].find('input');
        expect(medianInput.attributes('value')).toBe('median');
        expect(medianInput.attributes('checked')).not.toBeDefined();
    });

    it('expect that clicking the compare updates the store', async() => {
        const compareInput = wrapper.findAll('.usa-checkbox')[0].find('input');
        compareInput.element.checked = true;
        await compareInput.trigger('change');

        expect(store.getState().hydrographState.showCompareIVData).toBe(true);

        compareInput.element.checked = false;
        await compareInput.trigger('change');

        expect(store.getState().hydrographState.showCompareIVData).toBe(false);
    });

    it('expect that checking the compare fetches the prior year iv data and shows the data indicators', async() => {
        const compareInput = wrapper.findAll('.usa-checkbox')[0].find('input');
        compareInput.element.checked = true;
        await compareInput.trigger('change');

        expect(showDataIndicatorSpy).toHaveBeenCalledTimes(1);
        expect(showDataIndicatorSpy.mock.calls[0][0]).toBe(true);
        expect(retrievePriorYearSpy).toHaveBeenCalledTimes(1);
        expect(retrievePriorYearSpy.mock.calls[0][0]).toBe('12345678');
        expect(retrievePriorYearSpy.mock.calls[0][1]).toEqual({
            parameterCode: '72019',
            startTime: 1582560900000,
            endTime: 1600620300000
        });
    });

    it('expect that unchecking the compare does not fetch data and sets data indicators off', async() => {
        const compareInput = wrapper.findAll('.usa-checkbox')[0].find('input');
        compareInput.element.checked = false;
        await compareInput.trigger('change');

        expect(showDataIndicatorSpy).toHaveBeenCalledTimes(1);
        expect(showDataIndicatorSpy.mock.calls[0][0]).toBe(false);
        expect(retrievePriorYearSpy).not.toHaveBeenCalled();
    });

    it('expect that compare is disabled if selected time span is not one of the shortcut buttons', async() => {
        store.dispatch(setSelectedTimeSpan({
            start: '2020-02-05',
            end: '2020-02-10'
        }));
        await wrapper.vm.$nextTick();

        expect(wrapper.findAll('.usa-checkbox')[0].find('input').attributes('disabled')).toBeDefined();
    });

    it('expect that compare is enabled if a calculated temp parameter code is selected', async() => {
        store = configureStore({
            hydrographData: {
                currentTimeRange: TEST_CURRENT_TIME_RANGE
            },
            hydrographState: {
                showCompareIVData: false,
                selectedTimeSpan: 'P7D',
                showMedianData: false,
                selectedParameterCode: '00010F'
            }
        });

        wrapper = mount(GraphControls, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store,
                    siteno: '12345678'
                }
            }
        });

        expect(wrapper.findAll('.usa-checkbox')[0].find('input').attributes('disabled')).not.toBeDefined();
    });

    it('expect that clicking the median updates the store', async() => {
        const medianInput = wrapper.findAll('.usa-checkbox')[1].find('input');
        medianInput.element.checked = true;
        await medianInput.trigger('change');

        expect(store.getState().hydrographState.showMedianData).toBe(true);

        medianInput.element.checked = false;
        await medianInput.trigger('change');

        expect(store.getState().hydrographState.showMedianData).toBe(false);
    });

    it('expect that checking the median fetches the prior year iv data and shows the data indicators', async() => {
        const medianInput = wrapper.findAll('.usa-checkbox')[1].find('input');
        medianInput.element.checked = true;
        await medianInput.trigger('change');

        expect(showDataIndicatorSpy).toHaveBeenCalledTimes(1);
        expect(showDataIndicatorSpy.mock.calls[0][0]).toBe(true);
        expect(retrieveStatisticsSpy).toHaveBeenCalledTimes(1);
        expect(retrieveStatisticsSpy.mock.calls[0][0]).toBe('12345678');
        expect(retrieveStatisticsSpy.mock.calls[0][1]).toEqual('72019');
    });

    it('expect that unchecking the median does not fetch data and sets data indicators off', async() => {
        const medianInput = wrapper.findAll('.usa-checkbox')[1].find('input');
        medianInput.element.checked = false;
        await medianInput.trigger('change');

        expect(showDataIndicatorSpy).toHaveBeenCalledTimes(1);
        expect(showDataIndicatorSpy.mock.calls[0][0]).toBe(false);
        expect(retrieveStatisticsSpy).not.toHaveBeenCalled();
    });

    it('expects that if the secondary parameter is not null, the median and compare will be disabled', ()=> {
        store = configureStore({
            hydrographData: {
                currentTimeRange: TEST_CURRENT_TIME_RANGE
            },
            hydrographState: {
                showCompareIVData: false,
                selectedTimeSpan: 'P7D',
                showMedianData: false,
                selectedParameterCode: '72019',
                selectedSecondaryParameterCode: '00060'
            }
        });

        wrapper = mount(GraphControls, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store,
                    siteno: '12345678'
                }
            }
        });
        const compareInput = wrapper.findAll('.usa-checkbox')[0].find('input');
        const medianInput = wrapper.findAll('.usa-checkbox')[1].find('input');
        expect(compareInput.attributes().disabled).toBeDefined();
        expect(medianInput.attributes().disabled).toBeDefined();
    });
});