import {mount} from '@vue/test-utils';
import {bindActionCreators} from 'redux';
import ReduxConnectVue from 'redux-connect-vue';
import {createStructuredSelector} from 'reselect';

import config from 'ui/config';
import * as utils from 'ui/utils';

import {configureStore} from 'ml/store';

import {TEST_PRIMARY_IV_DATA, TEST_GW_LEVELS} from '../mock-hydrograph-state';

import HydrographLegend from './legend.vue';

describe('monitoring-location/components/hydrograph/legend module', () => {
    utils.mediaQuery = jest.fn().mockReturnValue(true);
    config.ivPeriodOfRecord = {
        '72019': {}
    };
    config.gwPeriodOfRecord = {
        '72019': {}
    };

    const DV_TEST_STATE = {
        dailyValueTimeSeriesData: {
            dvTimeSeries: {
                '12345': {
                    type: 'Feature',
                    id: '12345',
                    properties: {
                        phenomenonTimeStart: '2018-01-02',
                        phenomenonTimeEnd: '2018-01-05',
                        timeStep: ['2018-01-02', '2018-01-03', '2018-01-04', '2018-01-05'],
                        result: ['5.0', '4.0', '6.1', '3.2'],
                        approvals: [['Approved'], ['Approved'], ['Approved'], ['Approved']],
                        nilReason: [null, 'AA', null, null],
                        qualifiers: [null, ['ESTIMATED'], ['ICE'], ['ICE']],
                        grades: [['50'], ['50'], ['60'], ['60']]
                    }
                },
                '12346': {
                    type: 'Feature',
                    id: '12346',
                    properties: {
                        phenomenonTimeStart: '2018-01-02',
                        phenomenonTimeEnd: '2018-01-05',
                        timeStep: ['2018-01-02', '2018-01-03', '2018-01-04', '2018-01-05'],
                        result: ['6.0', '5.0', '7.1', '4.2'],
                        approvals: [['Approved'], ['Approved'], ['Approved'], ['Approved']],
                        nilReason: [null, 'AA', null, null],
                        qualifiers: [null, null, null, null],
                        grades: [['50'], ['50'], ['60'], ['60']]
                    }
                }
            }
        },
        dailyValueTimeSeriesState: {
            currentDVTimeSeriesId: {
                min: null,
                mean: '12345',
                max: '12346'
            }
        },
        ui: {
            windowWidth: 1024,
            width: 800
        }
    };

    const TEST_STATE = {
        hydrographData: {
            currentTimeRange: {
                start: 1582560000000,
                end: 1600700000000
            },
            primaryIVData: TEST_PRIMARY_IV_DATA
        },
        groundwaterLevelData: {
            all: [TEST_GW_LEVELS]
        },
        hydrographState: {
            showCompareIVData: false,
            showMedianData: false,
            selectedIVMethodID: '90649',
            selectedParameterCode: '72019'
        }
    };

    utils.mediaQuery = jest.fn().mockReturnValue(true);
    describe('HydrographLegend', () => {
        let store;
        let wrapper;

        beforeEach(() => {
            store = configureStore(TEST_STATE);

            wrapper = mount(HydrographLegend, {
                global: {
                    plugins: [
                        [ReduxConnectVue, {
                            store,
                            mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                            mapStateToPropsFactory: createStructuredSelector
                        }]
                    ]
                },
                props: {
                    graphType: 'IV'
                }
            });
        });

        it('Should have the correct number of legend markers', () => {
            expect(wrapper.findAll('.legend g')).toHaveLength(9);
        });

        it('Should have the correct number of legend markers when working for the DV graph', async() => {
            wrapper = mount(HydrographLegend, {
                global: {
                    plugins: [
                        [ReduxConnectVue, {
                            store: configureStore(DV_TEST_STATE),
                            mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                            mapStateToPropsFactory: createStructuredSelector
                        }]
                    ]
                },
                props: {
                    graphType: 'DV'
                }
            });
            await wrapper.vm.$nextTick();
            expect(wrapper.findAll('.legend g')).toHaveLength(6);
        });

        it('If no markers are provided legend-svg will contain no groups', async() => {
            wrapper = mount(HydrographLegend, {
                global: {
                    plugins: [
                        [ReduxConnectVue, {
                            store: configureStore({}),
                            mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                            mapStateToPropsFactory: createStructuredSelector
                        }]
                    ]
                },
                props: {
                    graphType: 'IV'
                }
            });
            await wrapper.vm.$nextTick();
            expect(wrapper.findAll('svg g')).toHaveLength(0);
        });

        it('Adds the correct number of legend elements', () => {
            expect(wrapper.findAll('svg')).toHaveLength(1);
            expect(wrapper.findAll('line')).toHaveLength(3);
            expect(wrapper.findAll('rect')).toHaveLength(2);
            expect(wrapper.findAll('text')).toHaveLength(9);
            expect(wrapper.findAll('circle')).toHaveLength(3);
        });

        it('Expects that the legend has the expected text', () => {
            let text = wrapper.findAll('text');
            expect(text[0].text()).toBe('Current:');
            expect(text[1].text()).toBe('Approved');
            expect(text[2].text()).toBe('Estimated');
            expect(text[3].text()).toBe('Provisional');
            expect(text[4].text()).toBe('Ice Affected');
            expect(text[5].text()).toBe('Field visit:');
            expect(text[6].text()).toBe('Provisional');
            expect(text[7].text()).toBe('Approved');
            expect(text[8].text()).toBe('Revised');
        });
    });
});