import {mount} from '@vue/test-utils';

import PaginatedTable from './paginated-table.vue';

describe('monitoring-location/components/hydrograph/components/data-table.vue', () => {
    let wrapper;
    const TEST_DATASET = [
        {
            time: '12:00',
            value: '3',
            notes: 'sth 456'
        },
        {
            time: '12:15',
            value: '453',
            notes: 'sth 1'
        },
        {
            time: '12:30',
            value: '123',
            notes: 'sth 2'
        },
        {
            time: '12:45',
            value: '35',
            notes: 'sth 3'
        },
        {
            time: '13:00',
            value: '33',
            notes: 'sth 4'
        },
        {
            time: '13:15',
            value: '1003',
            notes: 'sth 5'
        },
        {
            time: '13:30',
            value: '103',
            notes: 'sth 6'
        },
        {
            time: '13:45',
            value: '2',
            notes: 'sth 7'
        },
        {
            time: '14:00',
            value: '300',
            notes: 'sth 8'
        },
        {
            time: '14:15',
            value: '30',
            notes: 'sth 9'
        }
    ];

    it('Draws the table', () => {
        wrapper = mount(PaginatedTable, {
            props: {
                dataSet: TEST_DATASET,
                headers: ['Time', 'Result', 'Notes'],
                rowsPerPage: 2,
                title: 'Test Title',
                keyOrder: ['time', 'value', 'notes'],
                sortBy: ['time']
            }
        });
        const headers = wrapper.findAll('th');
        const rows = wrapper.findAll('tbody tr');
        expect(wrapper.find('caption').text()).toBe('Test Title');

        expect(headers).toHaveLength(3);
        expect(headers[0].text()).toBe('Time');
        expect(headers[1].text()).toBe('Result');
        expect(headers[2].text()).toBe('Notes');

        expect(rows).toHaveLength(2);
        expect(rows[0].findAll('td')).toHaveLength(3);
        expect(rows[0].findAll('td')[0].text()).toBe('14:15');
        expect(rows[0].findAll('td')[1].text()).toBe('30');
        expect(rows[0].findAll('td')[2].text()).toBe('sth 9');

        expect(rows[1].findAll('td')).toHaveLength(3);
        expect(rows[1].findAll('td')[0].text()).toBe('14:00');
        expect(rows[1].findAll('td')[1].text()).toBe('300');
        expect(rows[1].findAll('td')[2].text()).toBe('sth 8');

        expect(wrapper.findAll('nav')).toHaveLength(1);
    });

    it('Shows the pagination buttons', () => {
        wrapper = mount(PaginatedTable, {
            props: {
                dataSet: TEST_DATASET,
                headers: ['Time', 'Result', 'Notes'],
                rowsPerPage: 2,
                title: 'Test Title',
                keyOrder: ['time', 'value', 'notes'],
                sortBy: ['time']
            }
        });

        const pageButtons = wrapper.findAll('ul li');
        expect(pageButtons).toHaveLength(5);
        expect(pageButtons[0].text()).toBe('1');
        expect(pageButtons[1].text()).toBe('2');
        expect(pageButtons[2].text()).toBe('...');
        expect(pageButtons[3].text()).toBe('5');
        expect(pageButtons[4].html()).toContain('navigate_next');
    });

    it('Hides the pagination buttons if there is only one page', () => {
        wrapper = mount(PaginatedTable, {
            props: {
                dataSet: [
                    {
                        time: '12:00',
                        value: '3',
                        notes: 'sth 456'
                    },
                    {
                        time: '12:15',
                        value: '453',
                        notes: 'sth 1'
                    }],
                headers: ['Time', 'Result', 'Notes'],
                rowsPerPage: 2,
                title: 'Test Title',
                keyOrder: ['time', 'value', 'notes'],
                sortBy: ['time']
            }
        });

        const pageButtons = wrapper.findAll('ul li');
        expect(pageButtons).toHaveLength(0);
    });

    it('Changes the table when a page button is clicked', async() => {
        wrapper = mount(PaginatedTable, {
            props: {
                dataSet: TEST_DATASET,
                headers: ['Time', 'Result', 'Notes'],
                rowsPerPage: 2,
                title: 'Test Title',
                keyOrder: ['time', 'value', 'notes'],
                sortBy: ['time']
            }
        });
        const pageButtons = wrapper.findAll('ul li button');
        pageButtons[1].trigger('click');
        await wrapper.vm.$nextTick();
        const rows = wrapper.findAll('tbody tr');

        expect(rows[0].findAll('td')).toHaveLength(3);
        expect(rows[0].findAll('td')[0].text()).toBe('13:45');
        expect(rows[0].findAll('td')[1].text()).toBe('2');
        expect(rows[0].findAll('td')[2].text()).toBe('sth 7');

        expect(rows[1].findAll('td')).toHaveLength(3);
        expect(rows[1].findAll('td')[0].text()).toBe('13:30');
        expect(rows[1].findAll('td')[1].text()).toBe('103');
        expect(rows[1].findAll('td')[2].text()).toBe('sth 6');
    });

    it('Updates page buttons when page is changed', async() => {
        wrapper = mount(PaginatedTable, {
            props: {
                dataSet: TEST_DATASET,
                headers: ['Time', 'Result', 'Notes'],
                rowsPerPage: 2,
                title: 'Test Title',
                keyOrder: ['time', 'value', 'notes'],
                sortBy: ['time']
            }
        });

        let pageButtons = wrapper.findAll('ul li');
        expect(pageButtons).toHaveLength(5);
        expect(pageButtons[0].text()).toBe('1');
        expect(pageButtons[1].text()).toBe('2');
        expect(pageButtons[2].text()).toBe('...');
        expect(pageButtons[3].text()).toBe('5');

        wrapper.findAll('ul li button')[1].trigger('click');
        await wrapper.vm.$nextTick();
        pageButtons = wrapper.findAll('ul li');

        expect(pageButtons).toHaveLength(7);
        expect(pageButtons[1].text()).toBe('1');
        expect(pageButtons[2].text()).toBe('2');
        expect(pageButtons[3].text()).toBe('3');
        expect(pageButtons[4].text()).toBe('...');
        expect(pageButtons[5].text()).toBe('5');

        wrapper.findAll('ul li button')[0].trigger('click');
        await wrapper.vm.$nextTick();
        pageButtons = wrapper.findAll('ul li');

        expect(pageButtons).toHaveLength(5);
        expect(pageButtons[0].text()).toBe('1');
        expect(pageButtons[1].text()).toBe('2');
        expect(pageButtons[2].text()).toBe('...');
        expect(pageButtons[3].text()).toBe('5');
    });

    it('Sorts the table when the sort button is clicked', async() => {
        wrapper = mount(PaginatedTable, {
            props: {
                dataSet: TEST_DATASET,
                headers: ['Time', 'Result', 'Notes'],
                rowsPerPage: 2,
                title: 'Test Title',
                keyOrder: ['time', 'value', 'notes'],
                sortBy: ['time']
            }
        });
        const rows = wrapper.findAll('tbody tr');
        wrapper.find('svg').trigger('click');
        await wrapper.vm.$nextTick();

        expect(rows[0].findAll('td')).toHaveLength(3);
        expect(rows[0].findAll('td')[0].text()).toBe('12:00');
        expect(rows[0].findAll('td')[1].text()).toBe('3');
        expect(rows[0].findAll('td')[2].text()).toBe('sth 456');

        expect(rows[1].findAll('td')).toHaveLength(3);
        expect(rows[1].findAll('td')[0].text()).toBe('12:15');
        expect(rows[1].findAll('td')[1].text()).toBe('453');
        expect(rows[1].findAll('td')[2].text()).toBe('sth 1');

        wrapper.find('svg').trigger('click');
        await wrapper.vm.$nextTick();

        expect(rows).toHaveLength(2);
        expect(rows[0].findAll('td')).toHaveLength(3);
        expect(rows[0].findAll('td')[0].text()).toBe('14:15');
        expect(rows[0].findAll('td')[1].text()).toBe('30');
        expect(rows[0].findAll('td')[2].text()).toBe('sth 9');

        expect(rows[1].findAll('td')).toHaveLength(3);
        expect(rows[1].findAll('td')[0].text()).toBe('14:00');
        expect(rows[1].findAll('td')[1].text()).toBe('300');
        expect(rows[1].findAll('td')[2].text()).toBe('sth 8');
    });

    it('Resets the page count when the dataSet changes', async() => {
        wrapper = mount(PaginatedTable, {
            props: {
                dataSet: TEST_DATASET,
                headers: ['Time', 'Result', 'Notes'],
                rowsPerPage: 2,
                title: 'Test Title',
                keyOrder: ['time', 'value', 'notes'],
                sortBy: ['time']
            }
        });

        wrapper.findAll('ul li button')[3].trigger('click');
        await wrapper.vm.$nextTick();

        await wrapper.setProps({
            dataSet: [
                {
                    time: '12:00',
                    value: '3',
                    notes: 'sth 456'
                },
                {
                    time: '12:15',
                    value: '453',
                    notes: 'sth 1'
                },
                {
                    time: '12:30',
                    value: '123',
                    notes: 'sth 2'
                },
                {
                    time: '12:45',
                    value: '35',
                    notes: 'sth 3'
                },
                {
                    time: '13:00',
                    value: '33',
                    notes: 'sth 4'
                }]
        });

        const pageButtons = wrapper.findAll('ul li');
        expect(pageButtons).toHaveLength(4);
        expect(pageButtons[0].text()).toBe('1');
        expect(pageButtons[1].text()).toBe('2');
        expect(pageButtons[2].text()).toBe('3');
        expect(pageButtons[3].html()).toContain('navigate_next');
    });
});