import {enableFetchMocks, disableFetchMocks} from 'jest-fetch-mock';
import {bindActionCreators} from 'redux';
import ReduxConnectVue from 'redux-connect-vue';
import {createStructuredSelector} from 'reselect';
import {shallowMount} from '@vue/test-utils';

import config from 'ui/config';
import * as utils from 'ui/utils';

import {configureStore} from 'ml/store';
import * as hydrographData from 'ml/store/hydrograph-data';

import * as dataIndicator from '../data-indicator';
import {TEST_PRIMARY_IV_DATA, TEST_HYDROGRAPH_PARAMETERS} from '../mock-hydrograph-state';

import MethodPicker from './method-picker.vue';
import ParameterSelection from './parameter-selection.vue';
import ParameterSelectionExpansionControl from './parameter-selection-expansion-control.vue';
import SecondaryParameterControls from './secondary-parameter-controls.vue';


describe('monitoring-location/components/hydrograph/vue-components/parameter-selection', () => {
    utils.mediaQuery = jest.fn().mockReturnValue(true);

    config.SENSOR_THINGS_ENDPOINT = 'https://fake-sensor-things-endpoint';
    config.IV_DATA_ENDPOINT = 'https://fake-iv-data-endpoint';

    const TEST_STATE = {
        hydrographData: {
            primaryIVData: TEST_PRIMARY_IV_DATA
        },
        hydrographParameters: TEST_HYDROGRAPH_PARAMETERS,
        hydrographState: {
            selectedTimeSpan: 'P7D',
            selectedParameterCode: '72019',
            selectedIVMethodID: '90649'
        }
    };

    beforeAll(() => {
        enableFetchMocks();
    });

    afterAll(() => {
        disableFetchMocks();
    });

    let retrieveHydrographDataSpy;
    let showDataIndicatorSpy;
    let store;
    let wrapper;

    beforeEach(() => {

        config.ivPeriodOfRecord = {
            '00060': {
                begin_date: '1980-01-01',
                end_date: '2020-01-01'
            },
            '72019': {
                begin_date: '1980-04-01',
                end_date: '2020-04-01'
            },
            '00010': {
                begin_date: '1981-04-01',
                end_date: '2019-04-01'
            }
    
        };

        config.gwPeriodOfRecord = {
            '72019': {
                begin_date: '1980-03-31',
                end_date: '2020-03-31'
            },
            '62610': {
                begin_date: '1980-05-01',
                end_date: '2020-05-01'
            }
        };

        retrieveHydrographDataSpy = jest.spyOn(hydrographData, 'retrieveHydrographData');
        showDataIndicatorSpy = jest.spyOn(dataIndicator, 'showDataIndicators');
        store = configureStore(TEST_STATE);

        wrapper = shallowMount(ParameterSelection, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store,
                    siteno: '12345678',
                    agencyCode: 'USGS'
                }
            }
        });
    });

    it('Expects the component to be rendered with the expect rows and selection', () => {
        const rowContainers = wrapper.findAll('.parameter-row-container');

        expect(rowContainers).toHaveLength(5);
        expect(rowContainers[0].classes()).not.toContain('selected');
        expect(rowContainers[1].classes()).toContain('selected');
        expect(rowContainers[2].classes()).not.toContain('selected');
        expect(rowContainers[3].classes()).not.toContain('selected');
        expect(rowContainers[4].classes()).not.toContain('selected');

        expect(rowContainers[0].attributes('aria-selected')).toBe('false');
        expect(rowContainers[1].attributes('aria-selected')).toBe('true');
        expect(rowContainers[2].attributes('aria-selected')).toBe('false');
        expect(rowContainers[3].attributes('aria-selected')).toBe('false');
        expect(rowContainers[4].attributes('aria-selected')).toBe('false');
    });

    it('Expects a radio button and description for each parameter code with the selected parameter code checked', () =>{
        const radioButtons = wrapper.findAll('.usa-radio');

        expect(radioButtons).toHaveLength(5);

        const buttonOne = radioButtons[0].find('input');
        const labelOne = radioButtons[0].find('label');
        expect(buttonOne.element.checked).toBe(false);
        expect(buttonOne.attributes('value')).toBe('00060');
        expect(buttonOne.attributes('id')).toBe('radio-00060');
        expect(labelOne.text()).toBe('Discharge, cubic feet per second');
        expect(labelOne.attributes('for')).toBe('radio-00060');

        const buttonTwo = radioButtons[1].find('input');
        const labelTwo = radioButtons[1].find('label');
        expect(buttonTwo.element.checked).toBe(true);
        expect(buttonTwo.attributes('value')).toBe('72019');
        expect(buttonTwo.attributes('id')).toBe('radio-72019');
        expect(labelTwo.text()).toBe('Depth to water level, feet below land surface');
        expect(labelTwo.attributes('for')).toBe('radio-72019');
    });

    it('Expects the period of record and expansion toggle to be properly rendered', () => {
        const periodOfRecordContainers = wrapper.findAll('.period-of-record-container');

        expect(periodOfRecordContainers).toHaveLength(5);

        expect(periodOfRecordContainers[0].find('.period-of-record-text').text())
            .toBe('1980-01-01 to 2020-01-01');
        expect(periodOfRecordContainers[1].find('.period-of-record-text').text())
            .toBe('1980-03-31 to 2020-04-01');

        const expansionControls = wrapper.findAllComponents(ParameterSelectionExpansionControl);
        expect(expansionControls).toHaveLength(5);
        expect(expansionControls.find(control => control.props('parameterCode') === '62016')).toBeUndefined();

        const expansionControlOne = periodOfRecordContainers[0].findComponent(ParameterSelectionExpansionControl);
        const expansionControlTwo = periodOfRecordContainers[1].findComponent(ParameterSelectionExpansionControl);
        expect(expansionControlOne.props('parameterCode')).toBe('00060');
        expect(expansionControlTwo.props('parameterCode')).toBe('72019');
        expect(expansionControlOne.props('isExpanded')).toBe(false);
        expect(expansionControlTwo.props('isExpanded')).toBe(true);
        expect(expansionControlOne.props('idForExpansionRow')).toContain('00060');
        expect(expansionControlTwo.props('idForExpansionRow')).toContain('72019');
    });

    it('Expects that clicking on a row selects that parameter and closes any open row and opens that parameter\'s row', async() => {
        fetch.mockResponse(JSON.stringify({}));

        const parameterRowContainers = wrapper.findAll('.parameter-row-container');
        await parameterRowContainers[0].find('.parameter-row-info-container').trigger('click');

        expect(parameterRowContainers[0].classes('selected')).toBe(true);
        expect(parameterRowContainers[1].classes('selected')).toBe(false);
        expect(parameterRowContainers[0].findComponent(ParameterSelectionExpansionControl).props('isExpanded')).toBe(true);
        expect(parameterRowContainers[1].findComponent(ParameterSelectionExpansionControl).props('isExpanded')).toBe(false);
    });

    it('Expects that clicking on a row updates the selected parameter code in the state and fetches new hydrograph data', async() => {
        fetch.mockResponse(JSON.stringify({}));

        const parameterRowContainers = wrapper.findAll('.parameter-row-container');
        await parameterRowContainers[0].find('.parameter-row-info-container').trigger('click');

        expect(store.getState().hydrographState.selectedParameterCode).toBe('00060');
        expect(showDataIndicatorSpy).toHaveBeenCalled();
        expect(retrieveHydrographDataSpy).toHaveBeenCalled();
    });

    it('Expects the expansion container to be only shown for the expanded row on', () => {
        const expansionContainers = wrapper.findAll('.expansion-container-row');
        expect(expansionContainers).toHaveLength(5);
        expect(expansionContainers[0].isVisible()).toBe(false);
        expect(expansionContainers[1].isVisible()).toBe(true);
        expect(expansionContainers[2].isVisible()).toBe(false);
        expect(expansionContainers[3].isVisible()).toBe(false);
    });

    it('Expects a parameter code without methods shows only the water alert section', () => {
        const dischargeExpansionContainer = wrapper.find('#expansion-row-00060');
        const waterAlertLink = dischargeExpansionContainer.find('a');
        expect(waterAlertLink.text()).toBe( 'Subscribe to Alerts');
        expect(waterAlertLink.attributes('title')).toBeDefined();
        expect(dischargeExpansionContainer.findComponent(MethodPicker).exists()).toBe(false);
    });

    it('Expects a parameter code with methods shows the water alert section and the method picker', () => {
        const gwlevelsExpansionContainer = wrapper.find('#expansion-row-72019');
        const gwlevelsMethodPicker = gwlevelsExpansionContainer.findComponent(MethodPicker);
        expect(gwlevelsExpansionContainer.find('a').exists()).toBe(true);
        expect(gwlevelsMethodPicker.exists()).toBe(true);
        expect(gwlevelsMethodPicker.attributes('sortedivmethods')).toBeDefined();
    });

    it('Expects that clicking on a hidden expansion toggle shows the expansion container but does not change the selection', async() => {
        const expansionToggles = wrapper.findAllComponents(ParameterSelectionExpansionControl);
        await expansionToggles[0].vm.$emit('toggleExpansionRow', '00060', true);

        expect(wrapper.find('#expansion-row-00060').isVisible()).toBe(true);
        expect(wrapper.find('#expansion-row-72019').isVisible()).toBe(false);
        const parameterRows = wrapper.findAll('.parameter-row-container');
        expect(parameterRows[0].classes()).not.toContain('selected');
        expect(parameterRows[1].classes()).toContain('selected');
    });

    it('Expects that clicking the expansion toggle in the selected row hides the expansion container', async() => {
        const selectedExpansionToggle = wrapper.find('.selected').findComponent(ParameterSelectionExpansionControl);
        await selectedExpansionToggle.vm.$emit('toggleExpansionRow', '72019', false);

        expect(wrapper.find('#expansion-row-72019').isVisible()).toBe(false);
    });

    it('Expects that updated the selected method updates the store', async() => {
        const gwLevelMethodPicker = wrapper.find('#expansion-row-72019').findComponent(MethodPicker);
        await gwLevelMethodPicker.vm.$emit('selectMethod', '252055');

        expect(store.getState().hydrographState.selectedIVMethodID).toBe('252055');
    });

    it('Expects that the second parameter selection component will show only on selected parameter row with the correct props', () => {
        const parameterList =  [
            {
                'actionValue': {
                    'secondaryParameterCode': '00060'
                },
                'checked': false,
                'id': 'second-parameter-radio-button-00060',
                'label': 'Discharge, cubic feet per second'
            },
            {
                'actionValue': {
                    'secondaryParameterCode': '00010'
                },
                'checked': false,
                'id': 'second-parameter-radio-button-00010',
                'label': 'Temperature, water, degrees Celsius'
            },
            {
                'actionValue': {
                    'secondaryParameterCode': '00010F'
                },
                'checked': false,
                'id': 'second-parameter-radio-button-00010F',
                'label': 'Temperature, water, degrees Fahrenheit'
            }
        ];

        expect(wrapper.findAllComponents(SecondaryParameterControls)).toHaveLength(1);
        const secondaryParameterControl = wrapper.findComponent(SecondaryParameterControls);
        expect(secondaryParameterControl.props().parameterList).toStrictEqual(parameterList);
    });

    it('Expects that selecting a new parameter row will change the location of the second parameter selection component', async() => {
        fetch.mockResponse(JSON.stringify({}));

        const rowContainers = wrapper.findAll('.parameter-row-container');
        expect(rowContainers).toHaveLength(5);
        const rowForParameter00060 = rowContainers[0];
        const rowForParameter72019 = rowContainers[1];

        expect(rowForParameter00060.classes()).not.toContain('selected');
        expect(rowForParameter00060.findComponent(SecondaryParameterControls).exists()).toBe(false);
        expect(rowForParameter72019.classes()).toContain('selected');
        expect(rowForParameter72019.findComponent(SecondaryParameterControls).exists()).toBe(true);

        await rowForParameter00060.find('.parameter-row-info-container').trigger('click');
        expect(rowForParameter00060.classes()).toContain('selected');
        expect(rowForParameter00060.findComponent(SecondaryParameterControls).exists()).toBe(true);
        expect(rowForParameter72019.classes()).not.toContain('selected');
        expect(rowForParameter72019.findComponent(SecondaryParameterControls).exists()).toBe(false);
    });

    it('Expects that selecting a new parameter row without IV data will not show the second parameter selection component', async() => {
        fetch.mockResponse(JSON.stringify({}));

        const rowContainers = wrapper.findAll('.parameter-row-container');
        expect(rowContainers).toHaveLength(5);
        const rowForParameter62610 = rowContainers[2];
        const rowForParameter72019 = rowContainers[1];

        expect(rowForParameter62610.classes()).not.toContain('selected');
        expect(rowForParameter62610.findComponent(SecondaryParameterControls).exists()).toBe(false);
        expect(rowForParameter72019.classes()).toContain('selected');
        expect(rowForParameter72019.findComponent(SecondaryParameterControls).exists()).toBe(true);

        await rowForParameter62610.find('.parameter-row-info-container').trigger('click');
        expect(rowForParameter62610.classes()).toContain('selected');
        expect(rowForParameter62610.findComponent(SecondaryParameterControls).exists()).toBe(false);
        expect(rowForParameter72019.classes()).not.toContain('selected');
        expect(rowForParameter72019.findComponent(SecondaryParameterControls).exists()).toBe(false);
    });

    it('Expects that selecting a parameter row with IV data without any other IV data parameters available will not show the second parameter selection component', async() => {
        fetch.mockResponse(JSON.stringify({}));

        config.ivPeriodOfRecord = {
            '72019': {
                begin_date: '1980-04-01',
                end_date: '2020-04-01'
            }
        };

        const TEST_STATE = {
            hydrographParameters: TEST_HYDROGRAPH_PARAMETERS,
            hydrographState: {
                selectedTimeSpan: 'P7D',
                selectedParameterCode: '72019',
                selectedIVMethodID: '',
                selectedSecondaryParameterCode: ''
            }
        };

        store = configureStore(TEST_STATE);

        wrapper = shallowMount(ParameterSelection, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store,
                    siteno: '12345678',
                    agencyCode: 'USGS'
                }
            }
        });

        const rowContainers = wrapper.findAll('.parameter-row-container');
        expect(rowContainers).toHaveLength(5);
        const rowForParameter72019 = rowContainers[1];

        expect(rowForParameter72019.classes()).toContain('selected');
        expect(rowForParameter72019.findComponent(SecondaryParameterControls).exists()).toBe(false);
    });

    it('Expects that if no IV data is available the secondary parameter controls is not rendered for any row', () => {
        config.ivPeriodOfRecord = undefined;
        const TEST_STATE = {
            hydrographParameters: TEST_HYDROGRAPH_PARAMETERS,
            hydrographState: {
                selectedTimeSpan: 'P7D',
                selectedParameterCode: '72019',
                selectedIVMethodID: '',
                selectedSecondaryParameterCode: ''
            }
        };

        store = configureStore(TEST_STATE);

        wrapper = shallowMount(ParameterSelection, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store,
                    siteno: '12345678',
                    agencyCode: 'USGS'
                }
            }
        });

        expect(wrapper.findComponent(SecondaryParameterControls).exists()).toBe(false);
    });

    it('Expects events emitted from secondary parameter controls will change the secondary parameter code in the state', async() => {
        const TEST_STATE = {
            hydrographData: {
                primaryIVData: TEST_PRIMARY_IV_DATA
            },
            hydrographParameters: TEST_HYDROGRAPH_PARAMETERS,
            hydrographState: {
                selectedTimeSpan: 'P7D',
                selectedParameterCode: '72019',
                selectedIVMethodID: '90649',
                selectedSecondaryParameterCode: '00060'
            }
        };

        store = configureStore(TEST_STATE);

        wrapper = shallowMount(ParameterSelection, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store,
                    siteno: '12345678',
                    agencyCode: 'USGS'
                }
            }
        });
        expect(store.getState().hydrographState.selectedSecondaryParameterCode).toBe('00060');
        const secondaryParameterControls = wrapper.findComponent(SecondaryParameterControls);

        // Since the default state for checkbox is false, we need to emit twice; first setting to true, and then to false with clears the state
        await secondaryParameterControls.vm.$emit('setCheckBoxStatus');
        await secondaryParameterControls.vm.$emit('setCheckBoxStatus');
        expect(store.getState().hydrographState.selectedSecondaryParameterCode).toBe(null);
    });

    it('Expects that clicking a primary row with the same parameter code as the secondary parameter will remove the secondary parameter from the state', async() => {
        fetch.mockResponse(JSON.stringify({}));

        const TEST_STATE = {
            hydrographData: {
                primaryIVData: TEST_PRIMARY_IV_DATA
            },
            hydrographParameters: TEST_HYDROGRAPH_PARAMETERS,
            hydrographState: {
                selectedTimeSpan: 'P7D',
                selectedParameterCode: '72019',
                selectedIVMethodID: '90649',
                selectedSecondaryParameterCode: '00060'
            }
        };

        store = configureStore(TEST_STATE);

        wrapper = shallowMount(ParameterSelection, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store,
                    siteno: '12345678',
                    agencyCode: 'USGS'
                }
            }
        });

        const rowContainers = wrapper.findAll('.parameter-row-container');
        const rowForParameter00060 = rowContainers[0];
        const rowForParameter72019 = rowContainers[1];
        expect(store.getState().hydrographState.selectedSecondaryParameterCode).toBe('00060');
        expect(rowForParameter00060.classes()).not.toContain('selected');
        expect(rowForParameter72019.classes()).toContain('selected');

        await rowForParameter00060.find('.parameter-row-info-container').trigger('click');
        expect(store.getState().hydrographState.selectedSecondaryParameterCode).toBe(null);
    });
});
