import {enableFetchMocks, disableFetchMocks} from 'jest-fetch-mock';
import ReduxConnectVue from 'redux-connect-vue';
import {configureStore} from 'ml/store';
import {bindActionCreators} from 'redux';
import {createStructuredSelector} from 'reselect';
import {shallowMount} from '@vue/test-utils';

import {USWDSCheckbox} from '@wma/wdfn-vue-components';

import * as utils from 'ui/utils';

import USWDSRadioButtonSet from 'ui/uswds-components/radio-button-set.vue';

import {TEST_HYDROGRAPH_PARAMETERS, TEST_PRIMARY_IV_DATA} from '../mock-hydrograph-state';

import SecondaryParameterControls from './secondary-parameter-controls.vue';

describe('monitoring-location/components/hydrograph/vue-components/secondary-parameter-controls', () => {
    const parameterList =  [
        {
            'actionValue': {
                'secondaryParameterCode': '00060'
            },
            'checked': false,
            'id': 'second-parameter-radio-button-00060',
            'label': 'Discharge, cubic feet per second'
        },
        {
            'actionValue': {
                'secondaryParameterCode': '62610'
            },
            'checked': false,
            'id': 'second-parameter-radio-button-62610',
            'label': 'Groundwater level above NGVD 1929, feet'
        }
    ];

    beforeAll(() => {
        enableFetchMocks();
    });

    afterAll(() => {
        disableFetchMocks();
    });

    let wrapper;
    let store;

    const TEST_STATE = {
        hydrographData: {
            primaryIVData: TEST_PRIMARY_IV_DATA,
            currentTimeRange: {
                start: 1613413736967,
                end: 1614018536967
            }
        },
        hydrographParameters: TEST_HYDROGRAPH_PARAMETERS,
        hydrographState: {
            selectedTimeSpan: 'P7D',
            selectedParameterCode: '72019',
            selectedIVMethodID: '90649',
            selectedSecondaryIVMethodID: null,
            selectedSecondaryParameterCode: null,
            showMedianData: true,
            showCompareIVData: true
        }
    };

    describe('tests with check box checked', () => {
        beforeEach(() => {
            store = configureStore(TEST_STATE);
            wrapper = shallowMount(SecondaryParameterControls, {
                global: {
                    plugins: [
                        [ReduxConnectVue, {
                            store,
                            mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                            mapStateToPropsFactory: createStructuredSelector
                        }]
                    ],
                    provide: {
                        store: store,
                        siteno: '07144100',
                        agencyCode: 'USGS'
                    }
                },
                props: {
                    parameterList: parameterList,
                    isSecondParameterOptionsChecked: true
                }
            });
        });

        it('expects radio buttons will show if checkbox checked', () => {
            expect(wrapper.findAllComponents(USWDSCheckbox)).toHaveLength(1);
            expect(wrapper.findAllComponents(USWDSRadioButtonSet)).toHaveLength(1);
        });

        it('expects radio buttons will have correct props', () => {
            expect(wrapper.findAllComponents(USWDSCheckbox)).toHaveLength(1);
            expect(wrapper.findAllComponents(USWDSRadioButtonSet)).toHaveLength(1);

            const radioButtonSet = wrapper.findComponent(USWDSRadioButtonSet);
            expect(radioButtonSet.props('buttonList')).toStrictEqual(parameterList);
            expect(radioButtonSet.props('buttonSetName')).toBe('second-parameter-radio-button-set');
            expect(radioButtonSet.props('legendText')).toBe('Available secondary data types');
        });

        it('expects unchecking check box will emit checkbox event and not change the compare and median visibility', async() => {
            expect(store.getState().hydrographState.showMedianData).toBe(true);
            expect(store.getState().hydrographState.showCompareIVData).toBe(true);
            const checkboxComponent = wrapper.findComponent(USWDSCheckbox);
            await checkboxComponent.vm.$emit('toggleCheckbox');

            expect(wrapper.emitted().setCheckBoxStatus).toBeTruthy();
            expect(store.getState().hydrographState.showMedianData).toBe(true);
            expect(store.getState().hydrographState.showCompareIVData).toBe(true);
        });

        it('expects clicking a radio button will update the state for the secondary parameter', async() => {
            utils.mediaQuery = jest.fn().mockReturnValue(true);
            fetch.mockResponse(JSON.stringify({}));
            const radioButtonSetComponent = wrapper.findComponent(USWDSRadioButtonSet);
            expect(store.getState().hydrographState.selectedSecondaryParameterCode).toBe(null);
            expect(store.getState().hydrographState.showMedianData).toBe(true);
            expect(store.getState().hydrographState.showCompareIVData).toBe(true);


            await radioButtonSetComponent.vm.$emit('updateValue', {
                'secondaryParameterCode': '62610'
            });
            expect(store.getState().hydrographState.showMedianData).toBe(false);
            expect(store.getState().hydrographState.showCompareIVData).toBe(false);
            expect(store.getState().hydrographState.selectedSecondaryParameterCode).toBe('62610');
        });
    });

    describe('tests with check box not checked', () => {
        beforeEach(() => {
            store = configureStore(TEST_STATE);
            wrapper = shallowMount(SecondaryParameterControls, {
                global: {
                    plugins: [
                        [ReduxConnectVue, {
                            store,
                            mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                            mapStateToPropsFactory: createStructuredSelector
                        }]
                    ],
                    provide: {
                        store: store,
                        siteno: '07144100',
                        agencyCode: 'USGS'
                    }
                },
                props: {
                    parameterList: parameterList,
                    isSecondParameterOptionsChecked: false
                }
            });
        });

        it('Expects radio buttons will not show if checkbox not checked', () => {
            expect(wrapper.findAllComponents(USWDSCheckbox)).toHaveLength(1);
            expect(wrapper.findAllComponents(USWDSRadioButtonSet)).toHaveLength(0);
        });

        it('expects checking check box will emit correct events and payload', async() => {
            const checkboxComponent = wrapper.findComponent(USWDSCheckbox);

            await checkboxComponent.vm.$emit('toggleCheckbox');
            expect(wrapper.emitted('setCheckBoxStatus')).toBeTruthy();
        });
    });
});