import {bindActionCreators} from 'redux';
import ReduxConnectVue from 'redux-connect-vue';
import {createStructuredSelector} from 'reselect';
import {mount} from '@vue/test-utils';

import {configureStore} from 'ml/store';

import StatisticsTable from './statistics-table.vue';
import {TEST_STATS_DATA} from '../mock-hydrograph-state';

describe('monitoring-location/components/hydrograph/statistics', () => {

    describe('Cases with multiple methods available', () => {
        let store;
        let wrapper;

        beforeEach(() => {
            jest.useFakeTimers('modern');
            jest.setSystemTime(new Date(2020, 0, 1));

            store = configureStore({
                hydrographData: {
                    statisticsData: TEST_STATS_DATA,
                    primaryIVData: {
                        parameter: {
                            parameterCode: '153885',
                            name: 'Test Name'
                        },
                        values: {}
                    }
                },
                hydrographParameters: {
                    '153885': {
                        parameterCode: '153885',
                        name: 'Test Name',
                        latestValue: '25.9'
                    }
                },
                hydrographState: {
                    selectedParameterCode: '153885'
                }
            });

            wrapper = mount(StatisticsTable, {
                global: {
                    plugins: [
                        [ReduxConnectVue, {
                            store,
                            mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                            mapStateToPropsFactory: createStructuredSelector
                        }]
                    ]
                }
            });
        });

        afterEach(() => {
            jest.useRealTimers();
        });

        it('Displays the accordion', () => {
            expect(wrapper.find('.stats-accordion').isVisible()).toBe(true);
        });

        it('Creates multiple tables', () => {
            const captions = wrapper.findAll('caption');
            expect(wrapper.findAll('#daily-stats-tables div')).toHaveLength(2);
            expect(captions).toHaveLength(2);
            expect(captions[0].text()).toContain('Test Name (Method2)');
            expect(captions[1].text()).toContain('Test Name (Method1)');
        });

        it('Expects the table to have headers', () => {
            const tableHeaders = wrapper.find('thead').find('tr').findAll('tr > th');
            expect(tableHeaders).toHaveLength(6);
            expect(tableHeaders[0].text()).toBe('Lowest Value (2006)');
            expect(tableHeaders[1].text()).toBe('25th Percentile');
            expect(tableHeaders[2].text()).toBe('Median');
            expect(tableHeaders[3].text()).toBe('75th Percentile');
            expect(tableHeaders[4].text()).toBe('Mean');
            expect(tableHeaders[5].text()).toBe('Highest Value (2020)');
        });

        it('Expects the table to have data in it', () => {
            const tableHeaders = wrapper.find('tbody').find('tr').findAll('tr > th');
            expect(tableHeaders).toHaveLength(6);
            expect(tableHeaders[0].text()).toBe('550.5');
            expect(tableHeaders[1].text()).toBe('1000');
            expect(tableHeaders[2].text()).toBe('160');
            expect(tableHeaders[3].text()).toBe('2240');
            expect(tableHeaders[4].text()).toBe('1530');
            expect(tableHeaders[5].text()).toBe('2730');
        });
    });

    describe('Cases with one method available', () => {
        let store;
        let wrapper;

        beforeEach(() => {
            jest.useFakeTimers('modern');
            jest.setSystemTime(new Date(2020, 0, 1));

            store = configureStore({
                hydrographData: {
                    statisticsData: {'153885' : TEST_STATS_DATA['153885']},
                    primaryIVData: {
                        parameter: {
                            parameterCode: '153885',
                            name: 'Test Name'
                        },
                        values: {}
                    }
                },
                hydrographParameters: {
                    '153885': {
                        parameterCode: '153885',
                        name: 'Test Name',
                        latestValue: '25.9'
                    }
                },
                hydrographState: {
                    selectedParameterCode: '153885'
                }
            });

            wrapper = mount(StatisticsTable, {
                global: {
                    plugins: [
                        [ReduxConnectVue, {
                            store,
                            mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                            mapStateToPropsFactory: createStructuredSelector
                        }]
                    ]
                }
            });
        });

        afterEach(() => {
            jest.useRealTimers();
        });

        it('Displays the accordion', () => {
            expect(wrapper.find('.stats-accordion').isVisible()).toBe(true);
        });

        it('Creates multiple tables', () => {
            const captions = wrapper.findAll('caption');
            expect(wrapper.findAll('#daily-stats-tables div')).toHaveLength(1);
            expect(captions).toHaveLength(1);
            expect(captions[0].text()).toContain('Test Name (Method1)');
        });

        it('Expects the table to have headers', () => {
            const tableHeaders = wrapper.find('thead').find('tr').findAll('tr > th');
            expect(tableHeaders).toHaveLength(7);
            expect(tableHeaders[0].text()).toBe('Latest Value');
            expect(tableHeaders[1].text()).toBe('Lowest Value (2006)');
            expect(tableHeaders[2].text()).toBe('25th Percentile');
            expect(tableHeaders[3].text()).toBe('Median');
            expect(tableHeaders[4].text()).toBe('75th Percentile');
            expect(tableHeaders[5].text()).toBe('Mean');
            expect(tableHeaders[6].text()).toBe('Highest Value (2020)');
        });

        it('Expects the table to have data in it', () => {
            const tableHeaders = wrapper.find('tbody').find('tr').findAll('tr > th');
            expect(tableHeaders).toHaveLength(7);
            expect(tableHeaders[0].text()).toBe('25.9');
            expect(tableHeaders[1].text()).toBe('55.5');
            expect(tableHeaders[2].text()).toBe('100');
            expect(tableHeaders[3].text()).toBe('16');
            expect(tableHeaders[4].text()).toBe('224');
            expect(tableHeaders[5].text()).toBe('153');
            expect(tableHeaders[6].text()).toBe('273');
        });

    });

    describe('Cases with no stats data available', () => {
        let store;
        let wrapper;

        beforeEach(() => {
            jest.useFakeTimers('modern');
            jest.setSystemTime(new Date(2020, 0, 1));

            store = configureStore({
                hydrographData: {
                    statisticsData: {},
                    selectedParameterCode: '153885'
                },
                hydrographParameters: {
                    '153885': {
                        parameterCode: '153885',
                        name: 'Test Name',
                        latestValue: '25.9'
                    }
                }
            });

            wrapper = mount(StatisticsTable, {
                global: {
                    plugins: [
                        [ReduxConnectVue, {
                            store,
                            mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                            mapStateToPropsFactory: createStructuredSelector
                        }]
                    ]
                }
            });
        });

        afterEach(() => {
            jest.useRealTimers();
        });

        it('Does not show the table if there is no stats data', () => {
            expect(wrapper.find('.stats-accordion').isVisible()).toBe(false);
        });
    });

    describe('Cases with no latest value available', () => {
        let store;
        let wrapper;

        beforeEach(() => {
            jest.useFakeTimers('modern');
            jest.setSystemTime(new Date(2020, 0, 1));

            store = configureStore({
                hydrographData: {
                    statisticsData: TEST_STATS_DATA,
                    selectedParameterCode: '153885'
                },
                hydrographParameters: {
                    '153885': {
                        parameterCode: '153885',
                        name: 'Test Name'
                    }
                }
            });

            wrapper = mount(StatisticsTable, {
                global: {
                    plugins: [
                        [ReduxConnectVue, {
                            store,
                            mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                            mapStateToPropsFactory: createStructuredSelector
                        }]
                    ]
                }
            });
        });

        afterEach(() => {
            jest.useRealTimers();
        });

        it('Does not show the accordion if there is no latestValue', () => {
            expect(wrapper.find('.stats-accordion').isVisible()).toBe(false);
        });
    });
});