import {disableFetchMocks, enableFetchMocks} from 'jest-fetch-mock';
import mockConsole from 'jest-mock-console';

import {bindActionCreators} from 'redux';
import ReduxConnectVue from 'redux-connect-vue';
import {createStructuredSelector} from 'reselect';
import {shallowMount} from '@vue/test-utils';

import {USWDSAlert} from '@wma/wdfn-vue-components';

import USWDSDatePicker from 'ui/uswds-components/date-picker.vue';
import * as utils from 'ui/utils';

import {getGraphBrushOffset} from 'ml/selectors/hydrograph-state-selector';
import {getSelectedTimeSpan} from 'ml/selectors/hydrograph-state-selector';

import {configureStore} from 'ml/store';
import * as hydrographData from 'ml/store/hydrograph-data';
import {setSelectedTimeSpan} from 'ml/store/hydrograph-state';

import * as dataIndicator from '../data-indicator';

import TimeSpanControls from './time-span-controls.vue';




describe('monitoring-location/components/hydrograph/vue-components/time-span-controls.vue', () => {
    utils.mediaQuery = jest.fn().mockReturnValue(true);

    const TEST_STATE = {
        hydrographState: {
            selectedTimeSpan: 'P7D',
            selectedParameterCode: '00065',
            graphBrushOffset: {
                start: 0,
                end: 10000000000
            }
        }
    };

    let restoreConsole;

    beforeAll(() => {
        enableFetchMocks();
        restoreConsole = mockConsole();
    });

    afterAll(() => {
        disableFetchMocks();
        restoreConsole();
    });

    describe('tests for selectedTimeSpan with days time span', () => {
        let wrapper;
        let store;
        let showDataIndicatorSpy;
        let retrieveHydrographDataSpy;

        beforeEach(() => {
            store = configureStore(TEST_STATE);
            jest.useFakeTimers('modern');
            jest.setSystemTime(new Date(2021, 9, 1));

            retrieveHydrographDataSpy = jest.spyOn(hydrographData, 'retrieveHydrographData');
            showDataIndicatorSpy = jest.spyOn(dataIndicator, 'showDataIndicators');

            wrapper = shallowMount(TimeSpanControls, {
                global: {
                    plugins: [
                        [ReduxConnectVue, {
                            store,
                            mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                            mapStateToPropsFactory: createStructuredSelector
                        }]
                    ],
                    provide: {
                        store: store,
                        siteno: '12345678',
                        agencyCd: 'USGS'
                    }
                }
            });
        });

        afterEach(() => {
            jest.useRealTimers();
        });

        it('expects the correct elements to be rendered', () => {
            expect(wrapper.findAllComponents(USWDSDatePicker)).toHaveLength(2);
            expect(wrapper.findAll('input')).toHaveLength(1);
            expect(wrapper.findAll('button')).toHaveLength(1);
        });

        it('expects if the selectedTimeSpan has days, days will show in the days input', () => {
            const daysBeforeTodayInput = wrapper.find('#days-before-today');
            expect(daysBeforeTodayInput.exists()).toBe(true);
            expect(daysBeforeTodayInput.attributes('type')).toBe('text');
            expect(daysBeforeTodayInput.element.value).toBe('7');
        });

        it('expects USWDS date pickers are called with correct props', () => {
            const startDatePicker = wrapper.findAllComponents(USWDSDatePicker)[0];
            expect(startDatePicker.props('dateValue')).toBe('');
            expect(startDatePicker.props('maxDate')).toBe('2021-10-01');
            expect(startDatePicker.props('minDate')).toBe('1900-01-01');
            expect(startDatePicker.props('rangeDate')).toBe('');
            expect(startDatePicker.props('hintText')).toBe('mm/dd/yyyy');
            expect(startDatePicker.props('id')).toBe('start-date');
            expect(startDatePicker.props('label')).toBe('Start date');
            expect(startDatePicker.props('showLabel')).toBe(false);

            const endDatePicker = wrapper.findAllComponents(USWDSDatePicker)[1];
            expect(endDatePicker.props('dateValue')).toBe('');
            expect(endDatePicker.props('maxDate')).toBe('2021-10-01');
            expect(endDatePicker.props('minDate')).toBe('1900-01-01');
            expect(endDatePicker.props('rangeDate')).toBe('');
            expect(endDatePicker.props('hintText')).toBe('mm/dd/yyyy');
            expect(endDatePicker.props('id')).toBe('end-date');
            expect(endDatePicker.props('label')).toBe('End date');
            expect(endDatePicker.props('showLabel')).toBe(false);
        });

        it('expect that if days before today is changed and then the change time span button is clicked, new data is fetched', async() => {
            await wrapper.find('#days-before-today').setValue('20');
            await wrapper.find('.usa-button').trigger('click');

            const state = store.getState();
            expect(wrapper.findComponent(USWDSAlert).exists()).toBe(false);
            expect(getGraphBrushOffset(state)).toBeNull();
            expect(getSelectedTimeSpan(state)).toBe('P20D');
            expect(showDataIndicatorSpy).toHaveBeenCalledTimes(2);
            expect(showDataIndicatorSpy.mock.calls[0][0]).toBe(true);
            expect(showDataIndicatorSpy.mock.calls[1][0]).toBe(false);
            expect(retrieveHydrographDataSpy).toHaveBeenCalledTimes(1);
            expect(retrieveHydrographDataSpy.mock.calls[0][0]).toBe('12345678');
            expect(retrieveHydrographDataSpy.mock.calls[0][1]).toBe('USGS');
        });

        it('Expects to show an error message if days before today value is not a positive integer', async() => {
            await wrapper.find('#days-before-today').setValue('a');
            await wrapper.find('.usa-button').trigger('click');

            expect(wrapper.findComponent(USWDSAlert).exists()).toBe(true);
            expect(showDataIndicatorSpy).not.toHaveBeenCalled();
            expect(retrieveHydrographDataSpy).not.toHaveBeenCalled();

            await wrapper.find('#days-before-today').setValue('-1');
            await wrapper.find('.usa-button').trigger('click');

            expect(wrapper.findComponent(USWDSAlert).exists()).toBe(true);
            expect(showDataIndicatorSpy).not.toHaveBeenCalled();
            expect(retrieveHydrographDataSpy).not.toHaveBeenCalled();
        });
    });

    describe('tests for selectedTimeSpan with calendar time span', () => {
        let wrapper;
        let store;

        let showDataIndicatorSpy;
        let retrieveHydrographDataSpy;

        beforeEach(() => {
            store = configureStore({
                ...TEST_STATE,
                hydrographState: {
                    ...TEST_STATE.hydrographState,
                    selectedTimeSpan: {
                        'start': '2018-12-05',
                        'end': '2018-12-06'
                    }
                }
            });

            jest.useFakeTimers('modern');
            jest.setSystemTime(new Date(2021, 9, 1));

            retrieveHydrographDataSpy = jest.spyOn(hydrographData, 'retrieveHydrographData');
            showDataIndicatorSpy = jest.spyOn(dataIndicator, 'showDataIndicators');


            wrapper = shallowMount(TimeSpanControls, {
                global: {
                    plugins: [
                        [ReduxConnectVue, {
                            store,
                            mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                            mapStateToPropsFactory: createStructuredSelector
                        }]
                    ],
                    provide: {
                        store: store,
                        siteno: '12345678',
                        agencyCd: 'USGS'
                    }
                }
            });
        });

        afterEach(() => {
            jest.useRealTimers();
        });

        it('expects USWDS date pickers are called with correct props', () => {
            const startDatePicker = wrapper.findAllComponents(USWDSDatePicker)[0];
            expect(startDatePicker.props('dateValue')).toBe('2018-12-05');
            expect(startDatePicker.props('maxDate')).toBe('2018-12-06');
            expect(startDatePicker.props('minDate')).toBe('1900-01-01');
            expect(startDatePicker.props('rangeDate')).toBe('2018-12-06');

            const endDatePicker = wrapper.findAllComponents(USWDSDatePicker)[1];
            expect(endDatePicker.props('dateValue')).toBe('2018-12-06');
            expect(endDatePicker.props('maxDate')).toBe('2021-10-01');
            expect(endDatePicker.props('minDate')).toBe('2018-12-05');
            expect(endDatePicker.props('rangeDate')).toBe('2018-12-05');
        });

        it('expects that the days before today input will be blank', () => {
            const daysBeforeTodayInput = wrapper.find('#days-before-today');
            expect(daysBeforeTodayInput.exists()).toBe(true);
            expect(daysBeforeTodayInput.element.value).toBe('');
        });

        it('expects that if the user updates the start date picker that the data properties for the end picker are appropriately updated', async() => {
            const datePickers = wrapper.findAllComponents(USWDSDatePicker);
            await datePickers[0].vm.$emit('changeDate', '2018-12-06');

            expect(datePickers[1].props('minDate')).toBe('2018-12-06');
            expect(datePickers[1].props('maxDate')).toBe('2021-10-01');
            expect(datePickers[1].props('rangeDate')).toBe('2018-12-06');
        });

        it('expects that if the user updates the end date picker that the data properties for the start picker are appropriately updated', async() => {
            const datePickers = wrapper.findAllComponents(USWDSDatePicker);
            await datePickers[1].vm.$emit('changeDate', '2020-01-01');

            expect(datePickers[0].props('minDate')).toBe('1900-01-01');
            expect(datePickers[0].props('maxDate')).toBe('2020-01-01');
            expect(datePickers[0].props('rangeDate')).toBe('2020-01-01');
        });

        it('expects that if the user updates the time span and then clicks the change time span button that the selectedTimeSpan changes', async() => {
            const datePickers = wrapper.findAllComponents(USWDSDatePicker);
            const changeTimeSpanButton = wrapper.find('button');
            await datePickers[0].vm.$emit('changeDate', '2018-12-06');
            await datePickers[1].vm.$emit('changeDate', '2020-01-01');
            await changeTimeSpanButton.trigger('click');

            const state = store.getState();
            expect(wrapper.findComponent(USWDSAlert).exists()).toBe(false);
            expect(getGraphBrushOffset(state)).toBeNull();
            expect(getSelectedTimeSpan(state)).toEqual({
                start: '2018-12-06',
                end: '2020-01-01'
            });
            expect(showDataIndicatorSpy).toHaveBeenCalledTimes(2);
            expect(showDataIndicatorSpy.mock.calls[0][0]).toBe(true);
            expect(showDataIndicatorSpy.mock.calls[1][0]).toBe(false);
            expect(retrieveHydrographDataSpy).toHaveBeenCalledTimes(1);
            expect(retrieveHydrographDataSpy.mock.calls[0][0]).toBe('12345678');
            expect(retrieveHydrographDataSpy.mock.calls[0][1]).toBe('USGS');
        });

        it('Expects an error message if no time span or days before specified', async() => {
            await wrapper.find('#days-before-today').setValue('');
            const datePickers = wrapper.findAllComponents(USWDSDatePicker);
            await datePickers[0].vm.$emit('changeDate', '');
            await datePickers[1].vm.$emit('changeDate', '');
            await wrapper.find('.usa-button').trigger('click');

            expect(wrapper.findComponent(USWDSAlert).exists()).toBe(true);
            expect(showDataIndicatorSpy).not.toHaveBeenCalled();
            expect(retrieveHydrographDataSpy).not.toHaveBeenCalled();
        });

        it('Expects an error message if only start time or only end time is specified', async() => {
            const datePickers = wrapper.findAllComponents(USWDSDatePicker);
            await datePickers[0].vm.$emit('changeDate', '');
            await wrapper.find('.usa-button').trigger('click');

            expect(wrapper.findComponent(USWDSAlert).exists()).toBe(true);
            expect(showDataIndicatorSpy).not.toHaveBeenCalled();
            expect(retrieveHydrographDataSpy).not.toHaveBeenCalled();

            await datePickers[0].vm.$emit('changeDate', '2019-01-01');
            await datePickers[1].vm.$emit('changeDate', '');
            await wrapper.find('.usa-button').trigger('click');

            expect(wrapper.findComponent(USWDSAlert).exists()).toBe(true);
            expect(showDataIndicatorSpy).not.toHaveBeenCalled();
            expect(retrieveHydrographDataSpy).not.toHaveBeenCalled();
        });

        it('expects that if the start/end date pickers are filled out and the selectedTimeSpan is changed to a period the dates are cleared', async() => {
            store.dispatch(setSelectedTimeSpan('P7D'));
            await wrapper.vm.$nextTick();

            const datePickers = wrapper.findAllComponents(USWDSDatePicker);
            expect(datePickers[0].props('dateValue')).toBe('');
            expect(datePickers[1].props('dateValue')).toBe('');
        });
    });
});

