import {enableFetchMocks, disableFetchMocks} from 'jest-fetch-mock';
import mockConsole from 'jest-mock-console';
import {bindActionCreators} from 'redux';
import ReduxConnectVue from 'redux-connect-vue';
import {createStructuredSelector} from 'reselect';
import {mount} from '@vue/test-utils';

import config from 'ui/config';
import * as utils from 'ui/utils';

import {configureStore} from 'ml/store';
import * as hydrographData from 'ml/store/hydrograph-data';
import {setSelectedTimeSpan, setSelectedParameterCode} from 'ml/store/hydrograph-state';

import * as dataIndicator from '../data-indicator';

import {getSelectedTimeSpan, getGraphBrushOffset} from 'ml/selectors/hydrograph-state-selector';
import TimeSpanShortcuts from './time-span-shortcuts.vue';


const TEST_IV_PERIOD_OF_RECORD = {
    '00010': {
        begin_date: '2020-01-01',
        end_date: '2021-10-03'
    },
    '00060': {
        begin_date: '2019-01-01',
        end_date: '2021-10-01'
    },
    '72019': {
        begin_date: '2019-11-11',
        end_date: '2021-11-03'
    }
};

const TEST_GW_PERIOD_OF_RECORD = {
    '72019': {
        begin_date: '2020-01-01',
        end_date: '2021-11-04'
    },
    '62611': {
        begin_date: '2019-01-15',
        end_date: '2021-12-04'
    }
};



describe('monitoring-location/components/hydrograph/components/graph-controls', () => {
    utils.mediaQuery = jest.fn().mockReturnValue(true);
    config.ivPeriodOfRecord = {
        '72019': {}
    };

    let restoreConsole;
    let store;
    let showDataIndicatorSpy, retrieveSpy;
    let wrapper;
    config.ivPeriodOfRecord = TEST_IV_PERIOD_OF_RECORD;
    config.gwPeriodOfRecord = TEST_GW_PERIOD_OF_RECORD;

    beforeAll(() => {
        enableFetchMocks();
        restoreConsole = mockConsole();
    });

    afterAll(() => {
        disableFetchMocks();
        restoreConsole();
    });

    beforeEach(() => {
        retrieveSpy = jest.spyOn(hydrographData, 'retrieveHydrographData');
        showDataIndicatorSpy = jest.spyOn(dataIndicator, 'showDataIndicators');

        store = configureStore({
            hydrographState: {
                showCompareIVData: false,
                selectedTimeSpan: 'P7D',
                showMedianData: false,
                selectedParameterCode: '00060'
            }
        });

        wrapper = mount(TimeSpanShortcuts, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store,
                    siteno: '11112222',
                    agencyCd: 'USGS'
                }
            }
        });
    });

    it('Expects to render the shortcut radio buttons and check the 7 day radio button', () => {
        const radio7Day = wrapper.find('#P7D-input');
        const radio30Day = wrapper.find('#P30D-input');
        const radio1Year = wrapper.find('#P365D-input');
        const labels = wrapper.findAll('.usa-radio__label');

        expect(wrapper.findAll('.iv-button-container')).toHaveLength(1);
        expect(wrapper.findAll('.gw-button-container')).toHaveLength(0);

        expect(radio7Day.attributes('value')).toBe('P7D');
        expect(radio7Day.element.checked).toBe(true);
        expect(labels[0].text()).toBe('7 days');

        expect(radio30Day.attributes('value')).toBe('P30D');
        expect(radio30Day.element.checked).toBe(false);
        expect(labels[1].text()).toBe('30 days');

        expect(radio1Year.attributes('value')).toBe('P365D');
        expect(radio1Year.element.checked).toBe(false);
        expect(labels[2].text()).toBe('1 year');
    });

    it('Expects to render the shortcut radio buttons and check the 7 day radio button with a calculated temperature parameter code', () => {
        store = configureStore({
            hydrographState: {
                showCompareIVData: false,
                selectedTimeSpan: 'P7D',
                showMedianData: false,
                selectedParameterCode: '00010F'
            }
        });

        wrapper = mount(TimeSpanShortcuts, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store,
                    siteno: '11112222',
                    agencyCd: 'USGS'
                }
            }
        });

        const radio7Day = wrapper.find('#P7D-input');
        const radio30Day = wrapper.find('#P30D-input');
        const radio1Year = wrapper.find('#P365D-input');
        const labels = wrapper.findAll('.usa-radio__label');

        expect(wrapper.findAll('.iv-button-container')).toHaveLength(1);
        expect(wrapper.findAll('.gw-button-container')).toHaveLength(0);

        expect(radio7Day.attributes('value')).toBe('P7D');
        expect(radio7Day.element.checked).toBe(true);
        expect(labels[0].text()).toBe('7 days');

        expect(radio30Day.attributes('value')).toBe('P30D');
        expect(radio30Day.element.checked).toBe(false);
        expect(labels[1].text()).toBe('30 days');

        expect(radio1Year.attributes('value')).toBe('P365D');
        expect(radio1Year.element.checked).toBe(false);
        expect(labels[2].text()).toBe('1 year');
    });

    it('Expects that if the selectedTimeSpan is changed to a days before that is not a shortcut, they are all unset', async() => {
        store.dispatch(setSelectedTimeSpan('P45D'));
        await wrapper.vm.$nextTick();
         
        const radio7Day = wrapper.find('#P7D-input');
        const radio30Day = wrapper.find('#P30D-input');
        const radio1Year = wrapper.find('#P365D-input');

        expect(radio7Day.element.checked).toBe(false);
        expect(radio30Day.element.checked).toBe(false);
        expect(radio1Year.element.checked).toBe(false);
    });

    it('Expects that if the selectedTimeSpan is changed to a date range, the shortcut radio buttons are not checked', async() => {
        store.dispatch(setSelectedTimeSpan({start: '2020-01-03', end: '2020-05-01'}));
        await wrapper.vm.$nextTick();
        
        const radio7Day = wrapper.find('#P7D-input');
        const radio30Day = wrapper.find('#P30D-input');
        const radio1Year = wrapper.find('#P365D-input');

        expect(radio7Day.element.checked).toBe(false);
        expect(radio30Day.element.checked).toBe(false);
        expect(radio1Year.element.checked).toBe(false);
    });

    it('Expects that clicking a radio button updates the selectedTimeSpan and retrieve the data', async() => {
        const radio30Day = wrapper.find('#P30D-input');
        await radio30Day.trigger('click');

        const state = store.getState();
        expect(retrieveSpy).toHaveBeenCalledWith('11112222', 'USGS', {
            parameterCode: '00060',
            period: 'P30D',
            startTime: null,
            endTime: null,
            secondaryParameterCode: null,
            loadCompare: false
        });
        expect(showDataIndicatorSpy).toHaveBeenCalled();
        expect(getSelectedTimeSpan(state)).toEqual('P30D');
        expect(getGraphBrushOffset(state)).toBeNull();
    });

    it('Expects that clicking a radio button updates the selectedTimeSpan for the secondary parameter and retrieves the data', async() => {
        store = configureStore({
            hydrographState: {
                showCompareIVData: false,
                selectedTimeSpan: 'P7D',
                showMedianData: false,
                selectedParameterCode: '00060',
                selectedSecondaryParameterCode: '00065'
            }
        });

        wrapper = mount(TimeSpanShortcuts, {
            global: {
                plugins: [
                    [ReduxConnectVue, {
                        store,
                        mapDispatchToPropsFactory: (actionCreators) => (dispatch) => bindActionCreators(actionCreators, dispatch),
                        mapStateToPropsFactory: createStructuredSelector
                    }]
                ],
                provide: {
                    store: store,
                    siteno: '11112222',
                    agencyCd: 'USGS'
                }
            }
        });

        const radio30Day = wrapper.find('#P30D-input');
        await radio30Day.trigger('click');

        const state = store.getState();
        expect(retrieveSpy).toHaveBeenCalledWith('11112222', 'USGS', {
            parameterCode: '00060',
            period: 'P30D',
            startTime: null,
            endTime: null,
            secondaryParameterCode: '00065',
            loadCompare: false
        });
        expect(showDataIndicatorSpy).toHaveBeenCalled();
        expect(getSelectedTimeSpan(state)).toEqual('P30D');
        expect(getGraphBrushOffset(state)).toBeNull();
    });

    it('Expects that if the parameter code is changed to a gw only code the radio buttons will be different', async() => {
        store.dispatch(setSelectedParameterCode('62611'));
        await wrapper.vm.$nextTick();

        const radio1Year = wrapper.find('#P1Y-input');
        const radio10Year = wrapper.find('#P10Y-input');
        const radioAll = wrapper.find('#periodOfRecord-input');
        const labels = wrapper.findAll('.usa-radio__label');

        expect(wrapper.findAll('.iv-button-container')).toHaveLength(0);
        expect(wrapper.findAll('.gw-button-container')).toHaveLength(1);

        expect(radio1Year.attributes('value')).toBe('P1Y');
        expect(radio1Year.element.checked).toBe(true);
        expect(labels[0].text()).toBe('1 year');

        expect(radio10Year.attributes('value')).toBe('P10Y');
        expect(radio10Year.element.checked).toBe(false);
        expect(labels[1].text()).toBe('10 years');

        expect(radioAll.attributes('value')).toBe('periodOfRecord');
        expect(radioAll.element.checked).toBe(false);
        expect(labels[2].text()).toBe('Period of record');
    });

    it('Expects that clicking a gw radio button updates the selectedTimeSpan and retrieve the data', async() => {
        store.dispatch(setSelectedParameterCode('62611'));
        await wrapper.vm.$nextTick();

        const radio10Year = wrapper.find('#P10Y-input');
        await radio10Year.trigger('click');

        const state = store.getState();
        expect(retrieveSpy.mock.calls[0][2]).toEqual(expect.objectContaining({
            parameterCode: '62611',
            period: null,
            loadCompare: false
        }));
        expect(showDataIndicatorSpy).toHaveBeenCalled();
        expect(getSelectedTimeSpan(state)).toEqual('P10Y');
        expect(getGraphBrushOffset(state)).toBeNull();
    });
});