import {select} from 'd3-selection';
import {dynamicMapLayer, Util} from 'esri-leaflet';
import * as L from 'leaflet';
import {createStructuredSelector} from 'reselect';

import config from 'ui/config';
import {createMap, createBaseLayer} from 'ui/leaflet-rendering/map';
import {legendControl} from 'ui/leaflet-rendering/legend-control';
import {link} from 'ui/lib/d3-redux';

import {fetchSitesGeoJson} from 'ui/web-services/sensor-things';

import {hasFloodData, getFloodExtent, getFloodStageHeight} from 'ml/selectors/flood-data-selector';
import {hasNldiData, getNldiDownstreamFlows, getNldiUpstreamFlows, getNldiUpstreamBasin}
    from 'ml/selectors/nldi-data-selector';
import {Actions as nldiDataActions} from 'ml/store/nldi-data';
import {Actions as floodDataActions} from 'ml/store/flood-data';

import {floodSlider} from './flood-slider';
import {drawCircleMarkerLegend, drawFIMLegend, drawMonitoringLocationMarkerLegend} from './legend';
import {addNldiLayers, drawNldiLegend} from './nldi-mapping';

const ACTIVE_SITE_COLOR = 'red';
const ACTIVE_SITE_OPACITY = .8;
const geojsonMarkerOptions = {
    radius: 6,
    fillColor: ACTIVE_SITE_COLOR,
    color: '#000',
    weight: 1,
    opacity: 1,
    fillOpacity: ACTIVE_SITE_OPACITY
};

const getESRIFloodLayers = function(layerType) {
    return `${config.FIM_GIS_ENDPOINT}${layerType}/MapServer/`;
};

const getLayerDefs = function(layerNo, siteno, stage) {
   const stageQuery = stage ? ` AND STAGE = ${stage}` : '';
   return `${layerNo}: USGSID = '${siteno}'${stageQuery}`;
};

const getActiveMonitoringLocationsLayer = function(locations, markerOptions) {
    return L.geoJson(locations, {
        pointToLayer: function(feature, latlng) {
            return L.circleMarker(latlng, markerOptions);
        },
        onEachFeature: function(feature, layer) {
            const url = feature.properties['properties/monitoringLocationUrl'];
            const name = feature.properties['properties/monitoringLocationName'];
            const id = feature.properties['properties/monitoringLocationNumber'];
            const type = feature.properties.description;
            const popupText = `Monitoring Location: <a href="${url}">${name}</a>
                    <br/>ID: ${id}<br/>Site type: ${type}`;
            layer.bindPopup(popupText, {
                autoPan: false
            });
        }
    });
};

/*
 * Creates a site map
 */
const siteMap = function(node, {siteno, latitude, longitude, zoom}, store) {
    const map = createMap('site-map', {
        center: [latitude, longitude],
        zoom: zoom
    });
    
    const baseMapLayers = {
        'USGS Topo': createBaseLayer(config.TNM_USGS_TOPO_ENDPOINT),
        'Imagery': createBaseLayer(config.TNM_USGS_IMAGERY_ONLY_ENDPOINT),
        'Imagery+Topo': createBaseLayer(config.TNM_USGS_IMAGERY_TOPO_ENDPOINT)
    };

    const overlayLayers = {
        'Hydro': createBaseLayer(config.TNM_HYDRO_ENDPOINT)
    };

    map.addLayer(baseMapLayers['USGS Topo']);

    //add layer control
    L.control.layers(baseMapLayers, overlayLayers).addTo(map);

    // Add FIM layers
    const floodLayer = dynamicMapLayer({
        url: getESRIFloodLayers('floodExtents'),
        layers: [0],
        f: 'image',
        format: 'png8'
    });
    const breachLayer = dynamicMapLayer({
        url: getESRIFloodLayers('breach'),
        layers: [0],
        f: 'image',
        format: 'png8'
    });
    const leveeLayer = dynamicMapLayer({
        url: getESRIFloodLayers('suppLyrs'),
        layers: [0, 1],
        f: 'image',
        format: 'png8',
        layerDefs: `${getLayerDefs(0, siteno)};${getLayerDefs(1, siteno)}`
    });

    /*
     * Function to link to redux store to update the flood layers when flood data changes
     */
    const updateFloodLayers = function(node, {hasFloodData, floodStageHeight}) {
        if (floodStageHeight) {
            const layerDefs = getLayerDefs(0, siteno, floodStageHeight);
            floodLayer.setLayerDefs(layerDefs);
            breachLayer.setLayerDefs(layerDefs);
        }
        if (hasFloodData) {
            if (!map.hasLayer(floodLayer)) {
                map.addLayer(floodLayer);
                map.addLayer(breachLayer);
                map.addLayer(leveeLayer);
            }

        } else {
            if (map.hasLayer(floodLayer)) {
                map.removeLayer(floodLayer);
                map.removeLayer(breachLayer);
                map.removeLayer(leveeLayer);
            }
        }
    };

    /*
     * Function to link to Redux store to set the map extent when flood data is retrieved
     */
    const updateMapExtent = function(node, extent) {
        if (Object.keys(extent).length > 0) {
            map.fitBounds(Util.extentToBounds(extent).extend([latitude, longitude]));
        }
    };

    /*
     * Function to add the links for the Flood data information if flood data is available and/or
     * Nation Linked Data Index if monitoring location latitude and longitude available.
     */
    const addMapComponentLinks = function(node, hasFloodData) {
        node.select('#fim-link').remove();
        if (hasFloodData) {
            node.append('div')
                .attr('id', 'fim-link')
                .text('Interested in learning more about how a flood affects this area? ')
                .append('a')
                .classed('usa-link', true)
                .attr('href', `${config.FIM_ENDPOINT}?site_no=${siteno}`)
                .attr('target', '_blank')
                .attr('rel', 'noopener')
                .text('View this monitoring location on the Flood Information Inundation Mapper');
        }
        node.select('#nldi-link').remove();
        node.append('div')
            .attr('id', 'nldi-link')
            .text('Interested in understanding how to access the upstream/downstream data? ')
            .append('a')
            .classed('usa-link', true)
            .attr('href', 'https://waterdata.usgs.gov/blog/nldi-intro/')
            .attr('target', '_blank')
            .attr('rel', 'noopener')
            .text('Learn about the Network-Linked Data Index (NLDI)');
    };

    /*
     * Function to link to Redux store when NLDI data changes
     */
    const updateNldiLayers = function(node, {upstreamFlows, downstreamFlows, upstreamBasin}) {
        addNldiLayers(map, upstreamFlows, downstreamFlows, upstreamBasin);
    };

    // Create active site layer and function to update the sites layer with the current map bounds.
    let activeSitesLayer = L.layerGroup([]);
    const updateActiveSitesLayer = function(bounds) {
        const west = bounds.getWest().toPrecision(3);
        const south = bounds.getSouth().toPrecision(3);
        const east = bounds.getEast().toPrecision(3);
        const north = bounds.getNorth().toPrecision(3);
        const wktPolygon =
            `Polygon((${west} ${north}, ${west} ${south}, ${east} ${south}, ${east} ${north}, ${west} ${north}))`;

        activeSitesLayer.clearLayers();
        fetchSitesGeoJson(wktPolygon).then((featureCollection) => {
            if ('features' in featureCollection) {
                const locationsToDraw = featureCollection.features.filter(
                    (feature) => feature.properties['properties/monitoringLocationNumber'] !== siteno);
                activeSitesLayer.addLayer(getActiveMonitoringLocationsLayer(locationsToDraw, geojsonMarkerOptions));
            }
        });
    };

    map.addLayer(activeSitesLayer);
    updateActiveSitesLayer(map.getBounds());

    map.on('moveend', function() {
        updateActiveSitesLayer(map.getBounds());
    });

    // Add a marker at the site location
    L.marker([latitude, longitude]).addTo(map);

    // add legend and legend elements
    const mlLegendControl = legendControl();
    mlLegendControl.addTo(map);
    mlLegendControl.compressLegendOnSmallDevices();
    const legendListContainer = select(mlLegendControl.getContainer()).select('.legend-list-container');
    legendListContainer
        .call(drawMonitoringLocationMarkerLegend)
        .call(drawCircleMarkerLegend, ACTIVE_SITE_COLOR, ACTIVE_SITE_OPACITY, 'Active Monitoring Locations')
        .call(link(store, drawFIMLegend, hasFloodData))
        .call(link(store, drawNldiLegend, hasNldiData));

    // Connect the FIM information and NLDI information from the Redux Store to the
    // rendering code.
    node
        .call(link(store, updateFloodLayers, createStructuredSelector({
            hasFloodData: hasFloodData,
            floodStageHeight: getFloodStageHeight
        })))
        .call(link(store, updateMapExtent, getFloodExtent))
        .call(link(store, addMapComponentLinks, hasFloodData))
        .call(link(store, updateNldiLayers, createStructuredSelector({
            upstreamFlows: getNldiUpstreamFlows,
            downstreamFlows: getNldiDownstreamFlows,
            upstreamBasin: getNldiUpstreamBasin
        })));
};

/*
 * Creates the site map with node and attach it to the Redux store.
 * @param {Object} store - Redux store
 * @param {Object} node - DOM element
 * @param {String} siteno
 * @param {Number} latitude - latitude of siteno
 * @param {Number} longitude - longitude of siteno
 * @param {Number} zoom - zoom level to initially set the map to
 */
export const attachToNode = function(store, node, {siteno, latitude, longitude, zoom}) {
    store.dispatch(floodDataActions.retrieveFIMFloodData(siteno));
    // hydrates the store with nldi data
    store.dispatch(nldiDataActions.retrieveNldiData(siteno));

    select(node).select('#flood-layer-control-container')
        .call(floodSlider, store);
    select(node)
        .call(siteMap, {siteno, latitude, longitude, zoom}, store);
};
